# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2012-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import json
import logging

from django.conf import settings
from django.contrib import messages
from django.contrib.sites.models import Site
from django.http import HttpResponse, HttpResponseRedirect
from django.views.decorators.cache import never_cache
from django.views.generic import TemplateView

# https://github.com/jedie/django-tools
from django_tools.debug.delay import SessionDelay
from django_tools.middlewares.ThreadLocal import get_current_request


log = logging.getLogger(__name__)



@never_cache
def display_site(request):
    settings_id = settings.SITE_ID
    current_site = Site.objects.get_current()
    current_id = current_site.id

    txt = "ID from settings: %r - id from get_current(): %r" % (
        settings_id, current_id
    )
    log.debug("display_site(): %s", txt)
    return HttpResponse(txt)



def raise_exception(request, msg=""):
    """
    This view just raises an exception as a way to test middleware exception
    handling.
    """
    raise Exception(msg)


def get_current_get_parameters(request):
    """
    Returns a JSON version of request.GET from the current request
    """
    return HttpResponse(json.dumps(get_current_request().GET))


class TemplateDoesNotExists(TemplateView):
    template_name = "/template/does/not/exists.html"


def create_message_normal_response(request, msg):
    messages.info(request, msg)
    return HttpResponse("django_tools_test_project.django_tools_test_app.views.create_message_normal_response")


def create_message_redirect_response(request, msg):
    messages.info(request, msg)
    return HttpResponseRedirect("/create_message_redirect_response/")


def delay_view(request):
    """
    Used in django_tools_tests.test_debug_delay.SessionDelayTests
    """
    SessionDelay(
        request,
        key="delay_view",
        only_debug=False
    ).load(
        request,
        query_string="sec",
        default=5
    )

    SessionDelay(
        request,
        key="delay_view"
    ).sleep()

    return HttpResponse("django_tools_test_project.django_tools_test_app.views.delay_view")



# coding: utf-8

from __future__ import unicode_literals

import unittest

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.core.handlers.wsgi import WSGIRequest

from django_tools.utils.request import create_fake_request


class TestRequestUtils(unittest.TestCase):
    def test_defaults(self):
        fake_request = create_fake_request()
        self.assertIsInstance(fake_request, WSGIRequest)

        self.assertEqual(fake_request.path, "/")
        self.assertEqual(fake_request.session, {})
        self.assertEqual(fake_request.LANGUAGE_CODE, settings.LANGUAGE_CODE)
        self.assertIsInstance(fake_request.user, AnonymousUser)

    def test_change_defaults(self):
        class MockUser(object):
            pass
        mock_user = MockUser()

        fake_request = create_fake_request(
            url="/foo/bar",
            session={"foo":"bar"},
            language_code="es",
            user=mock_user,
            extra_attr1=True,
            extra_attr2=1234
        )
        self.assertIsInstance(fake_request, WSGIRequest)

        self.assertEqual(fake_request.path, "/foo/bar")
        self.assertEqual(fake_request.session, {"foo":"bar"})
        self.assertEqual(fake_request.LANGUAGE_CODE, "es")
        self.assertEqual(fake_request.user, mock_user)
        self.assertEqual(fake_request.extra_attr1, True)
        self.assertEqual(fake_request.extra_attr2, 1234)




import io
import sys

from django.utils.encoding import smart_text

# origin_stderr = sys.stderr

class StringBuffer(io.StringIO):
    def write(self, data):
        # origin_stderr.write("\nwrite to StringBuffer:\n%s\n\n" % repr(data))
        data = smart_text(data)
        super(StringBuffer, self).write(data)


class StdoutStderrBuffer():
    """
    redirect stderr and stdout for Py2 and Py3

    contextlib.redirect_stdout is new in Python 3.4!
    and we redirect stderr, too.

    We use django.utils.encoding.smart_text in StringBuffer()
    So output of bytes will only work, if there are encoded in UTF-8!

    e.g:

        with StdoutStderrBuffer() as buffer:
            print("foo")

        output = buffer.get_output()
    """
    def __init__(self):
        sys.stdout.flush()
        sys.stderr.flush()
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr

        self.buffer = StringBuffer()

        sys.stdout = sys.stderr = self.buffer

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.old_stdout.flush()
        self.old_stderr.flush()
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr

    def get_output(self):
        self.old_stdout.flush()
        self.old_stderr.flush()
        output = self.buffer.getvalue()
        return output






# coding: utf-8

"""
    need full validators
    ~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2010-2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function



import os
import re

from django.utils.six.moves import urllib
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator, URLValidator
from django.utils.translation import ugettext_lazy as _


# re from django.utils.translation.trans_real.accept_language_re
language_code_re = re.compile(r'^([A-Za-z]{1,8}(?:-[A-Za-z]{1,8})*)$')
validate_language_code = RegexValidator(
    language_code_re,
    _('Enter a valid language code (Accept-Language header format, see RFC2616)'),
    'invalid'
)


class ExistingDirValidator(object):
    def __init__(self, base_path=settings.MEDIA_ROOT):
        self.base_path = os.path.normpath(os.path.abspath(base_path))

    def __call__(self, value):
        abs_path = os.path.normpath(os.path.abspath(os.path.join(self.base_path, value)))

        if not abs_path.startswith(self.base_path):
            if settings.DEBUG:
                msg = _("Directory %r is not in base path ('%s')" % (abs_path, self.base_path))
            else:
                msg = _("Directory is not in base path!")
            raise ValidationError(msg)

        if not os.path.isdir(abs_path):
            if settings.DEBUG:
                msg = _("Directory %r doesn't exist!") % abs_path
            else:
                msg = _("Directory doesn't exist!")

            raise ValidationError(msg)

        return abs_path



class URLValidator2(URLValidator):
    """
    A flexible version of the original django URLValidator ;)
    
    scheme://netloc/path?query#fragment
    """
    regex = re.compile(r'^.+$', re.IGNORECASE)

    def __init__(self, allow_schemes=("http", "https"), allow_all_schemes=False, allow_netloc=True, allow_query=True, allow_fragment=True):
        super(URLValidator2, self).__init__()

        if __debug__ and (allow_schemes or allow_all_schemes) and not allow_netloc:
            raise AssertionError("Can't allow schemes without netloc!")

        if __debug__ and allow_schemes and allow_all_schemes:
            raise Warning("allow_schemes would be ignored, while allow_all_schemes==True!")

        self.allow_schemes = allow_schemes or ()
        self.allow_all_schemes = allow_all_schemes
        self.allow_netloc = allow_netloc
        self.allow_query = allow_query
        self.allow_fragment = allow_fragment

    def __call__(self, value):
        scheme, netloc, path, query, fragment = urllib.parse.urlsplit(value)

        if (scheme or netloc) and not self.allow_schemes and not self.allow_all_schemes and not self.allow_netloc:
            raise ValidationError(_("Please enter a local URL (without protocol/domain)."), code="local")

        if scheme:
            if not self.allow_all_schemes and scheme not in self.allow_schemes:
                raise ValidationError(_("The URL doesn't start with a allowed scheme."), "scheme")

        if netloc and not self.allow_netloc:
            raise ValidationError(_('Enter a valid URL without domain.'), code='netloc')

        if query and not self.allow_query:
            raise ValidationError(_('Enter a valid URL without a query.'), code='query')

        if fragment and not self.allow_fragment:
            raise ValidationError(_('Enter a valid URL without a fragment.'), code='fragment')



if __name__ == "__main__":
    import doctest
    print(doctest.testmod(
#        verbose=True
        verbose=False
    ))




from django.test import SimpleTestCase

# https://github.com/jedie/django-tools
from django_tools.utils.installed_apps_utils import get_filtered_apps


class TestGetFilteredApps(SimpleTestCase):
    def test_get_filtered_apps1(self):
        apps = get_filtered_apps()
        self.assertEqual(apps, [])

    def test_get_filtered_apps2(self):
        apps = get_filtered_apps(no_args=False)
        self.assertEqual(apps, ['django.contrib.flatpages'])

    def test_get_filtered_apps3(self):
        apps = get_filtered_apps(resolve_url="login/",
            # debug=True
        )
        self.assertEqual(apps, ['django.contrib.auth'])

    def test_get_filtered_apps4(self):
        apps = get_filtered_apps(resolve_url="login/", no_args=False)
        self.assertEqual(apps, ['django.contrib.auth', 'django.contrib.flatpages'])



# coding: utf-8

"""
    test django_tools.unittest_utils.user stuff
"""

from __future__ import unicode_literals

import warnings

import pytest

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.contrib.sessions.models import Session
from django.core.management import call_command
from django.test import TestCase

# https://github.com/jedie/django-tools
from django_tools.permissions import get_filtered_permissions
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer
from django_tools.unittest_utils.unittest_base import BaseTestCase
from django_tools.unittest_utils.user import (
    TestUserMixin, create_user, get_or_create_group, get_or_create_user, get_or_create_user_and_group, get_super_user
)


class TestUserUtils(TestUserMixin, BaseTestCase):

    def test_create_user(self):
        user1 = create_user(username="foo", password="bar")
        user2 = self.UserModel.objects.get(username="foo")

        self.assertEqual(user1.pk, user2.pk)
        self.assertEqual(user1, user2)

        # check defaults:
        self.assertFalse(user2.is_staff)
        self.assertFalse(user2.is_superuser)

    def test_superuser(self):
        create_user(username="foo", password="bar", is_superuser=True)
        user = self.UserModel.objects.get(username="foo")

        self.assertTrue(user.is_superuser)

        # check defaults:
        self.assertFalse(user.is_staff)

    def test_email(self):
        user = create_user(username="foo", password="foo", email="foo@bar.tld")
        self.assertEqual(user.email, "foo@bar.tld")

    def test_groups(self):
        group1 = Group.objects.create(name="Group 1")
        group2 = Group.objects.create(name="Group 2")
        create_user(username="foo", password="foo", groups=(group1, group2))
        user = self.UserModel.objects.get(username="foo")
        groups = user.groups.values_list('pk', 'name')
        self.assertEqual(list(groups), [(1, 'Group 1'), (2, 'Group 2')])

    def test_encrypted_password(self):
        user1 = create_user(username="foo1", password="bar")
        encrypted_password = user1.password
        create_user(username="foo2", encrypted_password=encrypted_password)
        user2 = self.UserModel.objects.get(username="foo2")
        self.assertEqual(user2.password, encrypted_password)

    def test_get_super_user(self):
        self.UserModel.objects.all().delete()

        create_user(username="foo", password="foo")
        user2 = create_user(username="bar", password="bar", is_superuser=True)
        user3 = get_super_user()

        self.assertEqual(user2.pk, user3.pk)
        self.assertEqual(user2, user3)
        self.assertTrue(user3.is_superuser)



class TestUserFixtures(TestUserMixin, BaseTestCase):
    def assert_user_fixtures(self):
        users = self.UserModel.objects.all()
        usernames = users.values_list("username", flat=True).order_by("username")
        reference = ('normal_test_user', 'staff_test_user', 'superuser')
        self.assertEqual(tuple(usernames), reference)

    def test_double_creation(self):
        self.assert_user_fixtures()
        self.create_testusers() # create user a second time
        self.assert_user_fixtures()

    def test_get_or_create_user_and_group(self):
        superuser = self.UserModel.objects.filter(is_superuser=True, is_active=True)[0]
        encrypted_password = superuser.password

        test_user = get_or_create_user_and_group(
            username="testuser",
            groupname="testgroup",
            permissions=get_filtered_permissions(
                exclude_app_labels=("admin", "sites"),
                exclude_models=(
                    Session,
                ),
                exclude_codenames=(
                    "delete_user",
                    "delete_group",
                ),
                exclude_permissions=(
                    (ContentType, "add_contenttype"),
                    (ContentType, "delete_contenttype"),
                )
            ),
            encrypted_password=encrypted_password
        )

        with StdoutStderrBuffer() as buff:
            call_command("permission_info", "testuser")
        output = buff.get_output()

        output = [line.strip(" \t_") for line in output.splitlines()]
        output = "\n".join([line for line in output if line])
        print(output)

        self.assertEqual_dedent(output, """
            Display effective user permissions in the same format as user.has_perm() argument: <appname>.<codename>
            All permissions for user 'testuser':
            is_active    : yes
            is_staff     : yes
            is_superuser : no
            [ ] admin.add_logentry
            [ ] admin.change_logentry
            [ ] admin.delete_logentry
            [*] auth.add_group
            [*] auth.change_group
            [ ] auth.delete_group
            [*] auth.add_permission
            [*] auth.change_permission
            [*] auth.delete_permission
            [*] auth.add_user
            [*] auth.change_user
            [ ] auth.delete_user
            [ ] contenttypes.add_contenttype
            [*] contenttypes.change_contenttype
            [ ] contenttypes.delete_contenttype
            [*] django_tools_test_app.add_limittousergroupstestmodel
            [*] django_tools_test_app.change_limittousergroupstestmodel
            [*] django_tools_test_app.delete_limittousergroupstestmodel
            [*] django_tools_test_app.add_permissiontestmodel
            [*] django_tools_test_app.change_permissiontestmodel
            [*] django_tools_test_app.delete_permissiontestmodel
            [*] django_tools_test_app.extra_permission
            [*] django_tools_test_app.add_simpleparlermodel
            [*] django_tools_test_app.change_simpleparlermodel
            [*] django_tools_test_app.delete_simpleparlermodel
            [*] easy_thumbnails.add_source
            [*] easy_thumbnails.change_source
            [*] easy_thumbnails.delete_source
            [*] easy_thumbnails.add_thumbnail
            [*] easy_thumbnails.change_thumbnail
            [*] easy_thumbnails.delete_thumbnail
            [*] easy_thumbnails.add_thumbnaildimensions
            [*] easy_thumbnails.change_thumbnaildimensions
            [*] easy_thumbnails.delete_thumbnaildimensions
            [*] filer.add_clipboard
            [*] filer.change_clipboard
            [*] filer.delete_clipboard
            [*] filer.add_clipboarditem
            [*] filer.change_clipboarditem
            [*] filer.delete_clipboarditem
            [*] filer.add_file
            [*] filer.change_file
            [*] filer.delete_file
            [*] filer.add_folder
            [*] filer.can_use_directory_listing
            [*] filer.change_folder
            [*] filer.delete_folder
            [*] filer.add_folderpermission
            [*] filer.change_folderpermission
            [*] filer.delete_folderpermission
            [*] filer.add_image
            [*] filer.change_image
            [*] filer.delete_image
            [*] filer.add_thumbnailoption
            [*] filer.change_thumbnailoption
            [*] filer.delete_thumbnailoption
            [*] flatpages.add_flatpage
            [*] flatpages.change_flatpage
            [*] flatpages.delete_flatpage
            [ ] sessions.add_session
            [ ] sessions.change_session
            [ ] sessions.delete_session
            [ ] sites.add_site
            [ ] sites.change_site
            [ ] sites.delete_site
        """)

        self.assertEqual(test_user.password, encrypted_password)

    def test_remove_obsolete_permissions(self):
        superuser = self.UserModel.objects.filter(is_superuser=True, is_active=True)[0]
        encrypted_password = superuser.password

        # Create with more permissions:
        get_or_create_user_and_group(
            username="testuser",
            groupname="testgroup",
            permissions=get_filtered_permissions(
                exclude_app_labels=("admin",),
                exclude_codenames=("delete_group",),
                exclude_permissions=(
                    (ContentType, "delete_contenttype"),
                )
            ),
            encrypted_password=encrypted_password
        )

        with StdoutStderrBuffer() as buff:
            get_or_create_user_and_group(
                username="testuser",
                groupname="testgroup",
                permissions=get_filtered_permissions(
                    exclude_app_labels=("admin", "sites"),
                    exclude_models=(
                        Session,
                    ),
                    exclude_codenames=(
                        "delete_user",
                        "delete_group",
                    ),
                    exclude_permissions=(
                        (ContentType, "add_contenttype"),
                        (ContentType, "delete_contenttype"),
                    )
                ),
                encrypted_password=encrypted_password
            )
        output = buff.get_output()
        print(output)
        self.assertEqual_dedent(output, """
            Check 'admin'
            Check 'sites'
            remove permission: auth | user | Can delete user
            remove permission: contenttypes | content type | Can add content type
            remove permission: sessions | session | Can add session
            remove permission: sessions | session | Can change session
            remove permission: sessions | session | Can delete session
            remove permission: sites | site | Can add site
            remove permission: sites | site | Can change site
            remove permission: sites | site | Can delete site
            Add 52 permissions to 'testgroup'
            Group testgroup has 52 permissions
        """)

    def test_update_existing_user(self):
        superuser = self.UserModel.objects.filter(is_superuser=True, is_active=True)[0]
        encrypted_password = superuser.password

        group1, created = get_or_create_group(groupname="group1", permissions=())
        self.assertTrue(created)

        user, created = get_or_create_user(
            username="foo",
            group=group1,
            encrypted_password=encrypted_password
        )
        self.assertTrue(created)

        self.assertEqual(
            list(self.UserModel.objects.get(username="foo").groups.all()),
            [group1]
        )

        # Update user and attach group2:

        group2, created = get_or_create_group(groupname="group2", permissions=())
        self.assertTrue(created)

        user, created = get_or_create_user(
            username="foo",
            group=group2,
            encrypted_password=encrypted_password
        )
        self.assertFalse(created)

        self.assertEqual(
            list(self.UserModel.objects.get(username="foo").groups.all()),
            [group2]
        )



# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2012-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
from django.conf import settings
from django.conf.urls import include, static, url
from django.contrib import admin

from django_tools_test_project.django_tools_test_app.views import (
    TemplateDoesNotExists, create_message_normal_response, create_message_redirect_response, display_site,
    get_current_get_parameters, raise_exception, delay_view
)

admin.autodiscover()


urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),

    url(r'^display_site/$', display_site),
    url(r'^get_current_get_parameters/$', get_current_get_parameters),
    url(r'^raise_exception/(?P<msg>.*?)/$', raise_exception),
    url(r'^raise_template_not_exists/$', TemplateDoesNotExists.as_view()),

    url(r'^create_message_normal_response/(?P<msg>.*?)/$', create_message_normal_response),
    url(r'^create_message_redirect_response/(?P<msg>.*?)/$', create_message_redirect_response),

    url(r'^delay/$', delay_view),
]

if settings.DEBUG:
    urlpatterns += static.static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    urlpatterns += static.static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

    import debug_toolbar
    urlpatterns = [
        url(r'^__debug__/', include(debug_toolbar.urls)),
    ] + urlpatterns



# coding: utf-8

"""
    template render
    ~~~~~~~~~~~~~~

    Some tools around template rendering

    :copyleft: 2009-2017 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

from django.template import Context, Template
from django.template.loader import get_template


def render_template_file(path, context):
    """
    Render given template file and return the rendered string
    """
    t = get_template(path)
    return t.render(context)


def render_string_template(template, context):
    """
    render the given template string with the context and return it as a string
    """
    t = Template(template)
    c = Context(context)
    return t.render(c)





# coding: utf-8

"""
    :copyleft: 2017 by the django-tools team, see AUTHORS for more details.
    :created: 2017 by Jens Diemer
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import print_function, unicode_literals

import os
from unittest import TestCase

from django.conf import settings
from django.core.management import call_command
from django.test import SimpleTestCase, override_settings

# https://github.com/jedie/django-tools
import django_tools
from django_tools.unittest_utils.django_command import DjangoCommandMixin
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer

MANAGE_DIR = os.path.abspath(os.path.join(os.path.dirname(django_tools.__file__), ".."))


class SettingsTests(SimpleTestCase):

    def test_settings_module(self):
        self.assertIn('django_tools_test_project.test_settings', settings.SETTINGS_MODULE)

    def test_diffsettings(self):
        """
        Check some settings
        """
        with StdoutStderrBuffer() as buff:
            call_command('diffsettings')
        output = buff.get_output()
        print(output)
        self.assertIn('django_tools_test_project.test_settings', output) # SETTINGS_MODULE


class ManageCommandTests(DjangoCommandMixin, TestCase):

    def test_help(self):
        """
        Run './manage.py --help' via subprocess and check output.
        """
        output = self.call_manage_py(['--help'], manage_dir=MANAGE_DIR)

        self.assertNotIn('ERROR', output)
        self.assertIn('[django]', output)
        self.assertIn('Type \'manage.py help <subcommand>\' for help on a specific subcommand.', output)

    # def test_unapplied_migrations(self):
    #     output = self.call_manage_py(["showmigrations"], manage_dir=MANAGE_DIR)
    #     print(output)
    #
    #     # We didn't have any migrations, yet:
    #     # self.assertIn('[X]', output) # applied migration
    #
    #     self.assertNotIn("[ ]", output) # unapplied migration

    def test_missing_migrations(self):
        output = self.call_manage_py(["makemigrations", "--dry-run"], manage_dir=MANAGE_DIR)
        print(output)
        self.assertIn("No changes detected", output)
        self.assertNotIn("Migrations for", output) # output like: """Migrations for 'appname':"""
        self.assertNotIn("SystemCheckError", output)
        self.assertNotIn("ERRORS", output)


class ManageCheckTests(SimpleTestCase):

    def test_django_check(self):
        """
        call './manage.py check' directly via 'call_command'
        """
        with StdoutStderrBuffer() as buff:
            call_command('check')
        output = buff.get_output()
        self.assertIn('System check identified no issues (0 silenced).', output)



# coding: utf-8

"""
    $ ./manage.py nice_diffsettings

    The code based on:
        django.core.management.commands.diffsettings
"""

from __future__ import absolute_import, unicode_literals, print_function

import pprint

from django.core.management.base import BaseCommand

def module_to_dict(module, omittable=lambda k: k.startswith('_')):
    """
    Same as django.core.management.commands.diffsettings.module_to_dict
    bur didn't use repr() ;)
    """
    return {k: v for k, v in module.__dict__.items() if not omittable(k)}


class Command(BaseCommand):
    help = """Displays differences between the current settings.py and Django's
    default settings in a pretty-printed representation."""

    requires_system_checks = False

    def add_arguments(self, parser):
        parser.add_argument('--all', action='store_true', dest='all', default=False,
            help='Display all settings, regardless of their value.')

    def handle(self, **options):
        from django.conf import settings, global_settings

        # Because settings are imported lazily, we need to explicitly load them.
        settings._setup()

        user_settings = module_to_dict(settings._wrapped)
        default_settings = module_to_dict(global_settings)

        self.stdout.write("-"*79)

        for key in sorted(user_settings):
            display = False
            if key not in default_settings:
                display = True
            elif user_settings[key] != default_settings[key]:
                display = True
            elif options['all']:
                display = True

            if display:
                self.stdout.write(
                    "%s = %s\n\n" % (key, pprint.pformat(user_settings[key]))
                )

        self.stdout.write("-" * 79)




from pprint import pprint

from django.conf import settings
from django.test import TestCase
from django.utils import translation

from django_tools_test_project.django_tools_test_app.models import generate_simple_parler_dummies, SimpleParlerModel


class ParlerFixturesTestCase(TestCase):
    def set_setup(self):
        self.assertEqual(settings.LANGUAGES, (('de', 'German'), ('en', 'English')))

    def test_generate_simple_parler_dummies(self):
        generate_simple_parler_dummies()

        self.assertEqual(SimpleParlerModel.objects.all().count(), 5)

        with translation.override("en"):
            qs = SimpleParlerModel.objects.language(language_code="en").all()

            info = []
            for instance in qs:
                info.append(
                    (instance.slug, list(instance.get_available_languages()))
                )

            pprint(info)

            # Every dummy entry should be translated in de and en:
            self.assertEqual(info, [
                ('simpleparlermodel-en-1', ['de', 'en']),
                ('simpleparlermodel-en-2', ['de', 'en']),
                ('simpleparlermodel-en-3', ['de', 'en']),
                ('simpleparlermodel-en-4', ['de', 'en']),
                ('simpleparlermodel-en-5', ['de', 'en'])
            ])

    # TODO:
    # def test_generate_parler_publisher_dummies(self):
    #     generate_parler_publisher_dummies()
    #
    #     self.assertEqual(ParlerPublisherModel.objects.all().count(), 5)
    #
    #     with translation.override("en"):
    #         qs = ParlerPublisherModel.objects.language(language_code="en").all().published()
    #
    #         info = []
    #         for instance in qs:
    #             info.append(
    #                 (instance.slug, list(instance.get_available_languages()))
    #             )
    #
    #         pprint(info)
    #
    #         # Every dummy entry should be translated in de and en:
    #         self.assertEqual(info, [
    #             ('simpleparlermodel-en-1', ['de', 'en']),
    #             ('simpleparlermodel-en-2', ['de', 'en']),
    #             ('simpleparlermodel-en-3', ['de', 'en']),
    #             ('simpleparlermodel-en-4', ['de', 'en']),
    #             ('simpleparlermodel-en-5', ['de', 'en'])
    #         ])



import warnings

from django.test import SimpleTestCase
from django.test.client import RequestFactory
from django.http import HttpResponse


from django_tools.utils.client_storage import SignedCookieStorage, SignedCookieStorageError, ClientCookieStorage



class TestSignedCookieStorage(SimpleTestCase):
    def test_signed_cookie1(self):

        response = HttpResponse("example")

        c = SignedCookieStorage("foo", max_age=123)
        response = c.save_data("bar", response)
        cookie = response.cookies["foo"]

        cookie_value = cookie.value
        # print(cookie_value) # e.g.: ImJhciI:1Z1xMG:XtRHzUuEg4xfiJPl-QeIbpWS3jM

        self.assertNotIn("bar", cookie_value)
        self.assertNotIn("foo", cookie_value)
        self.assertEqual(cookie["max-age"], 123)

        # print(response.cookies) # e.g.: Set-Cookie: foo=ImJhciI:1Z1y0f:wA2m4wjbUEwkS6TxK7gqZV9yk7M; expires=...
        self.assertIn("foo", response.cookies)

        request = RequestFactory().get('/', HTTP_COOKIE="foo=%s" % cookie_value)
        c = SignedCookieStorage("foo", max_age=123)
        self.assertEqual(c.get_data(request), "bar")

        # Try to access a not existing cookie:
        c = SignedCookieStorage("wrong name")
        self.assertRaises(SignedCookieStorageError, c.get_data, request)
        try:
            c.get_data(request)
        except SignedCookieStorageError as err:
            self.assertEqual(str(err), "Cookie 'wrong name' doesn't exists")

        # The existing, still there:
        c = SignedCookieStorage("foo", max_age=123)
        self.assertEqual(c.get_data(request), "bar")
    
    def test_wrong_data(self):
        request = RequestFactory().get('/', HTTP_COOKIE="foo=value:timestamp:wrong_dataABCDEFGHIJKLMNOPQ")
        c = SignedCookieStorage("foo")

        self.assertRaises(SignedCookieStorageError, c.get_data, request)
        try:
            c.get_data(request)
        except SignedCookieStorageError as err:
            self.assertEqual(str(err), """Can't load data: Signature "wrong_dataABCDEFGHIJKLMNOPQ" does not match""")

    def test_old_api(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always") # trigger all warnings

            c = ClientCookieStorage(cookie_key="foo")

            self.assertEqual(len(w), 1)
            self.assertEqual(str(w[-1].message), "ClientCookieStorage is old API! Please change to SignedCookieStorage! This will be removed in the future!")
            # self.assertIsInstance(w[-1].category, FutureWarning) # FIXME: AssertionError: <class 'FutureWarning'> is not an instance of <class 'FutureWarning'>
            self.assertTrue(issubclass(w[-1].category, FutureWarning))







# coding: utf-8

"""
    threadlocals middleware
    ~~~~~~~~~~~~~~~~~~~~~~~

    make the request object everywhere available (e.g. in model instance).

    based on: http://code.djangoproject.com/wiki/CookBookThreadlocalsAndUser

    Put this into your settings:
    --------------------------------------------------------------------------
        MIDDLEWARE_CLASSES = (
            ...
            'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
            ...
        )
    --------------------------------------------------------------------------


    Usage:
    --------------------------------------------------------------------------
    from django_tools.middlewares import ThreadLocal

    # Get the current request object:
    request = ThreadLocal.get_current_request()

    # You can get the current user directly with:
    user = ThreadLocal.get_current_user()
    --------------------------------------------------------------------------

    :copyleft: 2009-2017 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object  # fallback for Django < 1.10


_thread_locals = local()


def get_current_request():
    """ returns the request object for this thread """
    return getattr(_thread_locals, "request", None)


def get_current_user():
    """ returns the current user, if exist, otherwise returns None """
    request = get_current_request()
    if request:
        return getattr(request, "user", None)


class ThreadLocalMiddleware(MiddlewareMixin):
    """ Simple middleware that adds the request object in thread local storage."""
    def process_request(self, request):
        _thread_locals.request = request

    def process_response(self, request, response):
        if hasattr(_thread_locals, 'request'):
            del _thread_locals.request
        return response

    def process_exception(self, request, exception):
        if hasattr(_thread_locals, 'request'):
            del _thread_locals.request



"""
    :created: 2015 by Jens Diemer
    :copyleft: 2015-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging
from logging.handlers import MemoryHandler


class LoggingBuffer:

    def __init__(self, name=None, level=logging.DEBUG, formatter=None):
        """
        To get the logger name, execute this in `./manage.py shell` e.g.:

        import logging;print("\n".join(sorted(logging.Logger.manager.loggerDict.keys())))
        """
        self.buffer = []
        self.level = level
        if formatter is None:
            self.formatter = logging.Formatter(logging.BASIC_FORMAT)
        else:
            self.formatter = formatter

        self.log = logging.getLogger(name)
        self.old_handlers = self.log.handlers[:]  # .copy()
        self.old_level = self.log.level
        self.log.setLevel(level)
        self.log.handlers = [MemoryHandler(capacity=0, flushLevel=level, target=self)]

    def handle(self, record):
        self.buffer.append(record)

    def clear(self):
        self.buffer = []

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.log.handlers = self.old_handlers
        self.log.level = self.old_level

    def get_message_list(self):
        return [self.formatter.format(record) for record in self.buffer]

    def get_messages(self):
        return "\n".join(self.get_message_list())

    def assert_messages(self, reference):
        messages = self.get_message_list()
        assert messages == reference, "%r != %r" % (messages, reference)


class CutPathnameLogRecordFactory:
    """
    Adds 'cut_path' attribute on log record. So '%(cut_path)s' can be used in log formatter.

    Add this to you settings.py, e.g.:

        import logging
        from django_tools.unittest_utils.logging_utils import CutPathnameLogRecordFactory

        logging.setLogRecordFactory(CutPathnameLogRecordFactory(max_length=50))

        LOGGING = {
            # ...
            'formatters': {
                'verbose': {
                    'format': '%(levelname)8s %(cut_path)s:%(lineno)-3s %(message)s'
                },
            },
            # ...
        }
    """

    def __init__(self, max_length=40):
        self.max_length = max_length
        self.origin_factory = logging.getLogRecordFactory()

    def cut_path(self, pathname):
        if len(pathname) <= self.max_length:
            return pathname
        return "...%s" % pathname[-(self.max_length - 3):]

    def __call__(self, *args, **kwargs):
        record = self.origin_factory(*args, **kwargs)
        record.cut_path = self.cut_path(record.pathname)
        return record


class FilterAndLogWarnings:
    """
    Filter warnings and pipe them to logging system
    Warnings of external packages are displayed only once and only the file path.

    Activate warnings, e.g.:
        export PYTHONWARNINGS=all

    Add this to you settings.py, e.g.:

        import warnings
        import logging
        from django_tools.unittest_utils.logging_utils import FilterAndLogWarnings

        warnings.showwarning = FilterAndLogWarnings()
    """
    skipped_filenames = []

    def __init__(self, logger_name=None, external_package_paths=None):
        if logger_name is None:
            logger_name = __name__
        self.logger = logging.getLogger(logger_name)

        if external_package_paths is None:
            self.external_package_paths = (
                "/dist-packages/",
                "/site-packages/",
            )
        else:
            self.external_package_paths = external_package_paths

    def __call__(self, message, category, filename, lineno, args=None, exc_info=None):

        # handle warnings from external packages:
        for path_part in self.external_package_paths:
            if path_part in filename:
                if filename not in self.skipped_filenames:
                    self.skipped_filenames.append(filename)
                    self.logger.warning("There are warnings in: %s" % filename)
                return

        # Create log entry for the warning:
        record = self.logger.makeRecord(
            name=self.logger.name,
            level=logging.WARNING,
            fn=filename,
            lno=lineno,
            msg="%s:%s" % (category.__name__, message),
            args=args,
            exc_info=exc_info,
        )
        self.logger.handle(record)



import json

from django.test import TestCase

from django_tools.middlewares.ThreadLocal import get_current_request


class TestGetCurrentRequest(TestCase):
    def test_current_request_receives_current_request(self):
        response = self.client.get('/get_current_get_parameters/?')
        current_get_parameters = json.loads(response.content.decode('utf-8'))
        self.assertEqual(current_get_parameters, {})
        response = self.client.get('/get_current_get_parameters/?foo=bar')
        current_get_parameters = json.loads(response.content.decode('utf-8'))
        self.assertEqual(current_get_parameters, {'foo': 'bar'})

    def test_current_request_is_cleared_after_request_is_finished(self):
        response = self.client.get('/get_current_get_parameters/')
        self.assertEqual(get_current_request(), None)

    def test_current_request_is_cleared_when_exception_is_raised(self):
        with self.assertRaises(Exception):
            response = self.client.get('/raise_exception/TestGetCurrentRequest/')
        self.assertEqual(get_current_request(), None)




"""
    django-tools test models
    ~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2012-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.db import models

from parler.models import TranslatableModel, TranslatedFields

# https://github.com/jedie/django-tools
from django_tools import limit_to_usergroups
from django_tools.parler_utils.parler_fixtures import ParlerDummyGenerator
from django_tools.permissions import ModelPermissionMixin, check_permission


class LimitToUsergroupsTestModel(models.Model):
    permit_edit = limit_to_usergroups.UsergroupsModelField()
    permit_view = limit_to_usergroups.UsergroupsModelField()


class PermissionTestModel(ModelPermissionMixin, models.Model):
    foo = models.CharField(max_length=127)

    @classmethod
    def has_extra_permission_permission(cls, user, raise_exception=True):
        permission = cls.extra_permission_name(action="extra_permission")
        return check_permission(user, permission, raise_exception)

    class Meta:
        # https://docs.djangoproject.com/en/1.8/ref/models/options/#permissions
        permissions = (
            ("extra_permission", "Extra permission"),
        )


#_____________________________________________________________________________
# for django_tools.parler_utils.parler_fixtures.ParlerDummyGenerator


class SimpleParlerModel(TranslatableModel):
    translations = TranslatedFields(
        slug=models.SlugField()
    )


def generate_simple_parler_dummies():
    ParlerDummyGenerator(
        ParlerModelClass=SimpleParlerModel,
        publisher_model=False,
        unique_translation_field="slug"
    ).get_or_create(count=5)



#-----------------------------------------------------------------------------


# TODO:
# class ParlerPublisherModel(PublisherParlerModel):
#     shared = models.CharField(max_length=127)
#     translations = TranslatedFields(
#         foo=models.CharField(max_length=127)
#     )
#
#
# class ParlerPublisherModelFixtures(ParlerDummyGenerator):
#     def get_unique_translation_field_value(self, no, language_code):
#         return "%s %s no.%i" % (self.class_name, language_code, no)
#
#     def add_instance_values(self, instance, language_code, lang_name, no):
#         instance.shared="This is No.%i" % no
#         return instance
#
#
# def generate_parler_publisher_dummies():
#     ParlerPublisherModelFixtures(
#         ParlerModelClass=ParlerPublisherModel,
#         publisher_model=True,
#         unique_translation_field="foo"
#     ).get_or_create(count=3)



# coding: utf-8

"""
    filemanager
    ~~~~~~~~~~~

    Stuff to build a app like a file manager.


    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function



from operator import attrgetter
import datetime
import grp
import os
import pwd
import stat

if __name__ == "__main__":
    # For doctest only
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"
    from django.conf import global_settings
    global_settings.SITE_ID = 1

from django.contrib import messages

from django_tools.filemanager.exceptions import FilemanagerError
from django_tools.filemanager.filesystem_browser import BaseFilesystemBrowser
from django_tools.filemanager.utils import symbolic_notation


class BaseFilesystemObject(object):
    def __init__(self, base_path, name, abs_path, link_path=None):
        self.base_path = base_path # path in which this item exists
        self.name = name # The name of the directory item
        self.abs_path = abs_path # absolute path to this dir item
        self.link_path = link_path # Only for links: the real path of the dir item

        self.stat = os.stat(self.abs_path)
        self.size = self.stat[stat.ST_SIZE]
        self.mode = self.stat[stat.ST_MODE]
        self.mtime = datetime.datetime.fromtimestamp(self.stat[stat.ST_MTIME])

        self.mode_octal = oct(self.mode)
        self.mode_symbol = symbolic_notation(self.mode)

        self.uid = self.stat[stat.ST_UID]
        self.username = pwd.getpwuid(self.uid).pw_name
        self.gid = self.stat[stat.ST_GID]
        self.groupname = grp.getgrgid(self.gid).gr_name

    def __repr__(self):
        return "%s '%s' in %s" % (self.item_type, self.name, self.base_path)


class BaseFileItem(BaseFilesystemObject):
    is_file = True
    is_dir = False
    item_type = "file"


class BaseDirItem(BaseFilesystemObject):
    is_file = False
    is_dir = True
    item_type = "dir"


class BaseFileLinkItem(BaseFileItem):
    def __init__(self, *args, **kwargs):
        super(BaseFileLinkItem, self).__init__(*args, **kwargs)
        self.item_type = "file link to %s" % self.link_path

class BaseDirLinkItem(BaseDirItem):
    def __init__(self, *args, **kwargs):
        super(BaseDirLinkItem, self).__init__(*args, **kwargs)
        self.item_type = "dir link to %s" % self.link_path


#------------------------------------------------------------------------------


class BaseFilemanager(BaseFilesystemBrowser):
    """
    Base class for building a filemanager
    """
    DIR_ITEM = BaseDirItem
    FILE_ITEM = BaseFileItem
    DIR_LINK_ITEM = BaseDirLinkItem
    FILE_LINK_ITEM = BaseFileLinkItem

    def __init__(self, request, absolute_path, base_url, rest_url, allow_upload=False):
        super(BaseFilemanager, self).__init__(request, absolute_path, base_url, rest_url)

        self.allow_upload = allow_upload
        self.dir_items = self.read_dir(self.absolute_path)

    def read_dir(self, path):
        dir_items = []
        for item in os.listdir(path):
            item_abs_path = os.path.join(self.absolute_path, item)
            link_path = None
            if os.path.islink(item_abs_path):
                link_path = os.readlink(item_abs_path)
                if os.path.isdir(link_path):
                    item_class = self.DIR_LINK_ITEM
                elif os.path.isfile(link_path):
                    item_class = self.FILE_LINK_ITEM
                else:
                    raise NotImplemented
            elif os.path.isdir(item_abs_path):
                item_class = self.DIR_ITEM
            elif os.path.isfile(item_abs_path):
                item_class = self.FILE_ITEM
            else:
                messages.info(self.request, "unhandled directory item: %r" % self.absolute_path)
                continue

            instance = self.get_filesystem_item_instance(item_class, item, item_abs_path, link_path)
            dir_items.append(instance)

        # sort the dir items by name but directories first
        # http://wiki.python.org/moin/HowTo/Sorting/#Operator_Module_Functions
        dir_items = sorted(dir_items, key=attrgetter('item_type', 'name'))

        return dir_items

    def get_filesystem_item_instance(self, item_class, item, item_abs_path, link_path):
        """
        Good point for overwrite, to add attributes to the filesystem items.
        """
        instance = item_class(self.absolute_path, item, item_abs_path, link_path)
        return instance

    def handle_uploaded_file(self, f):
        if not self.allow_upload:
            raise FilemanagerError("Upload not allowed here!")

        path = os.path.join(self.absolute_path, f.name)
        destination = open(path, 'wb+')
        for chunk in f.chunks():
            destination.write(chunk)
        destination.close()

        messages.success(self.request,
            "File '%s' (%i Bytes) uploaded to %s" % (f.name, f.size, self.absolute_path)
        )


if __name__ == "__main__":
    import doctest
    print(doctest.testmod(
#        verbose=True
        verbose=False
    ))



# coding: utf-8

"""
    permission helpers
    ~~~~~~~~~~~~~~~~~~

    create 03.03.2017 by Jens Diemer
"""


from __future__ import absolute_import, print_function, unicode_literals

import logging

from django.conf import settings
from django.contrib.admin.sites import site
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied

log = logging.getLogger(__name__)


def pformat_permission(permission):
    """
    Generate the permission string in the same format as user.has_perm() argument:
        "<appname>.<codename>"
    """
    assert isinstance(permission, Permission), "No auth.models.Permission instance!"
    return "%s.%s" % (permission.content_type.app_label, permission.codename)


def get_permission(app_label, codename):
    try:
        perm_obj = Permission.objects.all().get(
            content_type__app_label=app_label,
            codename=codename
        )
    except Permission.DoesNotExist as err:
        log.exception("Error get permission '%s.%s':%s", app_label, codename, err)

        content_types = ContentType.objects.all().filter(app_label=app_label)
        if content_types.count() == 0:
            qs = ContentType.objects.all().values_list("app_label", flat=True).order_by("app_label")
            try:
                app_lables = ", ".join(qs.distinct("app_label"))
            except NotImplementedError:
                # e.g.: sqlite has no distinct
                app_lables = ", ".join(set(qs))
            raise PermissionDenied(
                (
                    "App label '%s' from permission '%s.%s' doesn't exists!"
                    " All existing labels are: %s"
                ) % (app_label, app_label, codename, app_lables)
            )
        qs = Permission.objects.all().filter(content_type__in=content_types)
        if qs.filter(codename=codename).count() == 0:
            codenames = qs.values_list("codename", flat=True).order_by("codename")
            raise PermissionDenied(
                (
                    "Codename '%s' from permission '%s.%s' doesn't exists!"
                    " All existing codenames are: %s"
                ) % (codename, app_label, codename, ", ".join(codenames))
            )
    else:
        return perm_obj


def get_permission_by_string(permission):
    try:
        app_label, codename = permission.split(".")
    except ValueError as err:
        raise PermissionDenied(
            "Wrong permission string format '%s': %s" % (permission, err)
        )
    return get_permission(app_label, codename)


def add_permissions(permission_obj, permissions):
    """
    Add permissions

    e.g.:
        group = Group.objects.create(name="Foobar Group")
        permissions=(
            (YourModelClass, "the_permission_string"),
        )
        add_permissions(permission_obj=group, permissions=permissions)
    """
    for model_class, codename in permissions:
        content_type = ContentType.objects.get_for_model(model_class)
        app_label = content_type.app_label
        permission = get_permission(app_label, codename)
        permission_obj.permissions.add(permission)


def permissions2list(permissions):
    """
    e.g.:

        permissions = our_user_group.permissions.all()
        permissions = permissions2list(permissions)
        pprint.pprint(permissions)
        ['auth.user.add_user',
         'auth.user.change_user',
         'auth.user.delete_user']
    """
    permissions = [pformat_permission(permission) for permission in permissions]
    return permissions


def log_user_permissions(user, log_callable=None):
    """
    Helper to log all existing user permissions.

    e.g.:
        from django_tools.permissions import log_group_permissions
        log = logging.getLogger(__name__)

        log_user_permissions(the_user_instance, log_callable=log.debug)
    """
    if log_callable is None:
        log_callable = log.debug

    permissions = sorted(user.get_all_permissions()) # A string list!

    if not permissions:
        log_callable("User '%s' has no permission!", user.username)
    else:
        permissions = "\n".join(["* %s" % p for p in permissions])
        log_callable("User '%s' has permissions:\n%s", user.username, permissions)


def log_group_permissions(group, log_callable=None):
    """
    Helper to log all existing user group permissions.

    e.g.:
        from django_tools.permissions import log_group_permissions
        log = logging.getLogger(__name__)

        log_group_permissions(user_group_instance, log_callable=log.debug)
    """
    if log_callable is None:
        log_callable = log.debug

    permissions = group.permissions.all().order_by("content_type", "codename")
    if not permissions:
        log_callable("User group '%s' has no permission!", group.name)
    else:
        permissions = "\n".join([
            "* %s" % pformat_permission(permission)
            for permission in permissions
        ])
        log_callable("User group '%s' has permissions:\n%s", group.name, permissions)


def get_admin_permissions():
    """
    :return: list of Permission instances for models that are registered in the admin
    """
    # Create a content type list of all models that are registered in the admin
    admin_content_types = []
    for model in site._registry.keys():
        content_type = ContentType.objects.get_for_model(model)
        admin_content_types.append(content_type)

    # Add all permissions for all admin models:
    permissions = Permission.objects.all()
    permissions = permissions.filter(content_type__in=admin_content_types)
    return permissions


def add_app_permissions(permission_obj, app_label):
    """
    Add all permission from one django app to the given object.
    e.g.:
        add_app_permissions(permission_obj=user_group_instance, app_label="filer")
    """
    content_types = ContentType.objects.filter(app_label = app_label)
    permissions = Permission.objects.filter(content_type__in=content_types)
    log.debug("Add %i permissions from app '%s'" % (permissions.count(), app_label))
    for permission in permissions:
        permission_obj.permissions.add(permission)


def check_permission(user, permission, raise_exception=True):
    """
    Check if given user has the given permission

    :param user: django user model instance
    :param permission: string e.g.: 'auth.user.add_user'
    :param raise_exception: raise PermissionDenied if user has not the permission
    :return: bool if user has permission
    """
    if user.is_superuser:
        log.debug("Don't check permissions for superuser, ok.")
        return True

    if user.has_perm(permission):
        log.debug('User "%s" has permission "%s"', user, permission)
        return True

    if settings.DEBUG:
        # 'validate' permission string
        # get_permission() will raise helpfull errors if format is wrong
        # or if the permission doesn't exists
        get_permission_by_string(permission)

    if raise_exception:
        log.error(
            'User "%s" has not permission "%s" -> raise PermissionDenied!',
            user, permission
        )
        raise PermissionDenied
    else:
        log.debug('User "%s" has not permission "%s"', user, permission)
        return False


def has_perm(user, permission):
    """
    Verbose check if user has permission:
    Create log entry if user has not the permission
    """
    if not user.has_perm(permission):
        log.debug('User %s has not %s', user, permission)
        return False
    return True


class ModelPermissionMixin(object):
    """
    Helper for easy model permission checks.
    e.g.:
        class FooModel(ModelPermissionMixin, models.Model):
            ...
            @classmethod
            def has_extra_permission_permission(cls, user, raise_exception=True):
                permission = cls.extra_permission_name(action="extra_permission")
                return check_permission(user, permission, raise_exception)

            class Meta:
                permissions = (
                    ("extra_permission", "Extra permission"),
                )

        def view(request):
            FooModel.has_change_permission(request.user, raise_exception=True)
            FooModel.has_extra_permission_permission(request.user, raise_exception=True)

    See also model used in our tests:
        django_tools_test_project.django_tools_test_app.models.PermissionTestModel
    """
    @classmethod
    def default_permission_name(cls, action):
        """
        Built the permission name for the default add/change/delete permissions.

        Note: Django will add "model name" to these permissions!
        e.g.:
            applabel.add_modelname
            applabel.change_modelname
            applabel.delete_modelname

        https://docs.djangoproject.com/en/1.8/ref/models/options/#default-permissions
        """
        assert action in cls._meta.default_permissions, "'%s' not in Meta.default_permissions !" % action
        permission = "{app}.{action}_{model}".format(
            app=cls._meta.app_label,
            action=action,
            model=cls._meta.model_name,
        )
        return permission

    @classmethod
    def extra_permission_name(cls, action):
        """
        Built permission name for own/extra permissions defined by "ModelClass.Meta.permissions"

        Note: Django will create these permissions without "model name"!
        e.g.:
            applabel.permission_name_modelname
        """
        if settings.DEBUG:
            all_permissions = [p[0] for p in cls._meta.permissions]
            assert action in all_permissions, "'%s' not in Meta.permissions! Existing keys are: %s" % (
                action, ", ".join(all_permissions)
            )
        permission = "{app}.{action}".format(
            app=cls._meta.app_label,
            action=action,
        )
        return permission

    @classmethod
    def has_add_permission(cls, user, raise_exception=True):
        permission = cls.default_permission_name(action="add")
        return check_permission(user, permission, raise_exception)

    @classmethod
    def has_change_permission(cls, user, raise_exception=True):
        permission = cls.default_permission_name(action="change")
        return check_permission(user, permission, raise_exception)

    @classmethod
    def has_delete_permission(cls, user, raise_exception=True):
        permission = cls.default_permission_name(action="delete")
        return check_permission(user, permission, raise_exception)


def get_filtered_permissions(
        exclude_app_labels=None,
        exclude_actions=None,
        exclude_models=None,
        exclude_codenames=None,
        exclude_permissions=None
    ):
    """
    Generate a Permission instance list and exclude parts of it.

    Filters:
        <app_label>.<action>_<modelname>
                    `-----codename-----'

    usage, e.g.:
        permissions = get_filtered_permissions(
            exclude_app_labels=("easy_thumbnails", "filer"),
            exclude_actions=("delete",)
            exclude_models=(LimitToUsergroupsTestModel, PermissionTestModel),
            exclude_codenames=("add_group", "add_user"),
            exclude_permissions=(
                (ContentType, "add_contenttype"),
                (ContentType, "delete_contenttype"),
            )
        )
    """
    qs = Permission.objects.all().order_by("content_type__app_label", "content_type__model", "codename")

    if exclude_codenames is not None:
        qs = qs.exclude(codename__in=exclude_codenames)

    if exclude_actions is not None:
        for action in exclude_actions:
            codename_prefix = "%s_" % action
            qs = qs.exclude(codename__startswith=codename_prefix)

    if exclude_app_labels is not None:
        # SQLite doesn't support .distinct, so we can't do this:
        # app_lables = ContentType.objects.all().values_list("app_label", flat=True).order_by("app_label").distinct("app_label")
        # This isn't perfomance criticle code, isn't it? So just do this:
        app_lables = set(ContentType.objects.all().values_list("app_label", flat=True))

        exclude_content_types = []
        for app_label in exclude_app_labels:
            if app_label not in app_lables:
                raise AssertionError(
                    "app label %r not found! Existing labels: %s" % (
                        app_label, ",".join(app_lables)
                    )
                )
            print("Check %r" % app_label)
            content_types = ContentType.objects.all().filter(app_label = app_label)
            assert content_types.count()>0
            exclude_content_types += content_types

        qs = qs.exclude(content_type__in=exclude_content_types)

    if exclude_models is not None:
        for model in exclude_models:
            content_type = ContentType.objects.get_for_model(model)
            qs = qs.exclude(content_type=content_type)

    if exclude_permissions is not None:
        for model, codename in exclude_permissions:
            content_type = ContentType.objects.get_for_model(model)
            qs = qs.exclude(content_type=content_type, codename=codename)

    return qs


def pprint_filtered_permissions(permissions):
    """
    print a list like this:
        [*] auth.group.add_group
        [*] auth.group.change_group
        [ ] auth.group.delete_group
        [*] auth.permission.add_permission
        [ ] auth.permission.change_permission
        [*] auth.permission.delete_permission
        [*] auth.user.add_user
        [*] auth.user.change_user
        [ ] auth.user.delete_user
        ...
    """
    for permission in permissions:
        assert isinstance(permission, Permission), "List must contain auth.models.Permission instances!"

    qs = Permission.objects.all().order_by("content_type__app_label", "content_type__model", "codename")
    for permission in qs:
        contains = "[*]" if permission in permissions else "[ ]"
        print("%s %s" % (contains, pformat_permission(permission)))



# coding: utf-8

"""
    per-site cache middleware
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    more information in the README.

    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import logging
import sys

from django.conf import settings
from django.contrib import messages
from django.core.cache import cache
from django.http import HttpResponse
from django.utils.cache import get_max_age, patch_response_headers

from django_tools.utils.importlib import get_attr_from_settings


logger = logging.getLogger("django-tools.CacheMiddleware")

CACHE_MIDDLEWARE_ANONYMOUS_ONLY = getattr(settings, 'CACHE_MIDDLEWARE_ANONYMOUS_ONLY', False)
RUN_WITH_DEV_SERVER = getattr(settings, "RUN_WITH_DEV_SERVER", "runserver" in sys.argv)
EXTRA_DEBUG = getattr(settings, "CACHE_EXTRA_DEBUG", False)

COUNT_FETCH_FROM_CACHE = getattr(settings, "COUNT_FETCH_FROM_CACHE", False)
COUNT_UPDATE_CACHE = getattr(settings, "COUNT_UPDATE_CACHE", False)
COUNT_IN_CACHE = getattr(settings, "COUNT_IN_CACHE", False)

cache_callback = get_attr_from_settings("CACHE_CALLBACK", "DjangoTools cache callback")
logger.debug("Use cache callback: %s" % repr(cache_callback))


CACHE_REQUESTS = "DJANGOTOOLS_CACHE_REQUESTS_COUNT"
CACHE_REQUEST_HITS = "DJANGOTOOLS_CACHE_REQUEST_HITS_COUNT"
CACHE_RESPONSES = "DJANGOTOOLS_CACHE_RESPONSES_COUNT"
CACHE_RESPONSE_HITS = "DJANGOTOOLS_CACHE_RESPONSE_HITS_COUNT"

_CACHE_KEYS = (CACHE_REQUESTS, CACHE_REQUEST_HITS, CACHE_RESPONSES, CACHE_RESPONSE_HITS)


# Some information from the cache usage.
# Note: These values are only for current process/thread valid and they not thread safe!
# Set settings.COUNT_IN_CACHE=True for process/thread safe counting.
LOCAL_CACHE_INFO = {
    # from FetchFromCacheMiddleware:
    "requests": None, # total numbers of requests
    "request hits": None, # number of cache hits

    # from UpdateCacheMiddleware:
    "responses": None, # total numbers of responses
    "response hits": None, # number of responses from cache
}


def init_cache_counting():
    """
    prepare LOCAL_CACHE_INFO and the counter in cache:
    All initial count value should be None, if count is disabled
    and should be 0, if enabled.
    """
    for key in _CACHE_KEYS:
        cache.delete(key) # delete old entrie, if exist

    if COUNT_FETCH_FROM_CACHE:
        # Count in FetchFromCacheMiddleware is enabled
        LOCAL_CACHE_INFO["requests"] = 0
        LOCAL_CACHE_INFO["request hits"] = 0
        if COUNT_IN_CACHE:
            # Count in cache is enabled
            cache.set(CACHE_REQUESTS, 0)
            cache.set(CACHE_REQUEST_HITS, 0)

    if COUNT_UPDATE_CACHE:
        # Count in UpdateCacheMiddleware is enabled
        LOCAL_CACHE_INFO["responses"] = 0
        LOCAL_CACHE_INFO["response hits"] = 0
        if COUNT_IN_CACHE:
            # Count in cache is enabled
            cache.set(CACHE_RESPONSES, 0)
            cache.set(CACHE_RESPONSE_HITS, 0)

init_cache_counting()


def build_cache_key(url, language_code, site_id):
    cache_key = "%s:%s:%s" % (url, language_code, site_id)
    if EXTRA_DEBUG:
        logger.debug("Cache key: %r" % cache_key)
    return cache_key


def get_cache_key(request):
    """
    Build the cache key based on the url and:

    * LANGUAGE_CODE: The language code in the url can be different than the
        used language for gettext translation.
    * SITE_ID: request.path is the url without the domain name. So the same
        url in site A and B would result in a collision.
    """
    url = request.get_full_path()

    try:
        language_code = request.LANGUAGE_CODE # set in django.middleware.locale.LocaleMiddleware
    except AttributeError:
        etype, evalue, etb = sys.exc_info()
        evalue = etype("%s (django.middleware.locale.LocaleMiddleware must be insert before cache middleware!)" % evalue)
        raise etype(evalue).with_traceback(etb)

    site_id = settings.SITE_ID
    cache_key = build_cache_key(url, language_code, site_id)
    return cache_key


def delete_cache_item(url, language_code, site_id=None):
    if site_id is None:
        site_id = settings.SITE_ID

    cache_key = build_cache_key(url, language_code, site_id)
    logger.debug("delete from cache: %r" % cache_key)
    cache.delete(cache_key)


class CacheMiddlewareBase(object):
    def use_cache(self, request, response=None):
        if not request.method in ('GET', 'HEAD'):
            logger.debug("Don't cache %r (%s)" % (request.method, request.get_full_path()))
            return False

        if RUN_WITH_DEV_SERVER and request.path.startswith(settings.STATIC_URL):
            if EXTRA_DEBUG:
                logger.debug("Don't cache static files in dev server")
            return False

        if response and response.status_code != 200:
            logger.debug("Don't cache response with status code: %s (%s)" % (response.status_code, request.get_full_path()))
            return False

        if CACHE_MIDDLEWARE_ANONYMOUS_ONLY and request.user.is_authenticated():
            logger.debug("Don't cache requests from authenticated users.")
            return False

        if hasattr(request, '_messages') and len(request._messages) != 0:
            msg = "Don't cache: page for anonymous users has messages."
            if settings.DEBUG:
                storage = messages.get_messages(request)
                raw_messages = ", ".join([message.message for message in storage])
                storage.used = False
                msg += " (messages: %s)" % raw_messages
            logger.debug(msg)
            return False

        if response and getattr(response, 'csrf_processing_done', False):
            logger.debug("Don't cache because response.csrf_processing_done==True (e.g.: view use @csrf_protect decorator)")
            return False

        if cache_callback:
            return cache_callback(request, response)

        return True


def save_incr(key, default=1):
    try:
        cache.incr(key)
    except ValueError: # Doesn't exist, yet.
        cache.set(key, default)


class FetchFromCacheMiddleware(CacheMiddlewareBase):
    def _count_requests(self, request):
        if RUN_WITH_DEV_SERVER and request.path.startswith(settings.STATIC_URL):
            return

        LOCAL_CACHE_INFO["requests"] += 1
        if COUNT_IN_CACHE:
            save_incr(CACHE_REQUESTS)

    def _count_hit(self):
        LOCAL_CACHE_INFO["request hits"] += 1
        if COUNT_IN_CACHE:
            save_incr(CACHE_REQUEST_HITS)

    def process_request(self, request):
        """
        Try to fetch response from cache, if exists.
        """
        if COUNT_FETCH_FROM_CACHE:
            self._count_requests(request)

        if not self.use_cache(request):
            if EXTRA_DEBUG:
                logger.debug("Don't fetch from cache: %s" % request.get_full_path())
            return

        cache_key = get_cache_key(request)
        response = cache.get(cache_key)
        if response is None:
            logger.debug("Not found in cache: %r" % cache_key)
        else:
            logger.debug("Use %r from cache!" % cache_key)
            if COUNT_FETCH_FROM_CACHE:
                self._count_hit()
            response._from_cache = True
            return response


class UpdateCacheMiddleware(CacheMiddlewareBase):
    def _count_response(self, request):
        if RUN_WITH_DEV_SERVER and request.path.startswith(settings.STATIC_URL):
            return

        LOCAL_CACHE_INFO["responses"] += 1
        if COUNT_IN_CACHE:
            save_incr(CACHE_RESPONSES)

    def _count_hit(self):
        LOCAL_CACHE_INFO["response hits"] += 1
        save_incr(CACHE_RESPONSE_HITS)

    def process_response(self, request, response):
        if COUNT_UPDATE_CACHE:
            self._count_response(request)

        if getattr(response, "_from_cache", False) == True:
            if COUNT_UPDATE_CACHE:
                self._count_hit()
            logger.debug("response comes from the cache, no need to update the cache")
            return response
        else:
            # used e.g. in unittests
            response._from_cache = False

        if not self.use_cache(request, response):
            if EXTRA_DEBUG:
                logger.debug("Don't put to cache: %s" % request.get_full_path())
            return response

        # get the timeout from the "max-age" section of the "Cache-Control" header
        timeout = get_max_age(response)
        if timeout == None:
            # use default cache_timeout
            timeout = settings.CACHE_MIDDLEWARE_SECONDS
        elif timeout == 0:
            logger.debug("Don't cache this page (timeout == 0)")
            return response

        # Create a new HttpResponse for the cache, so we can skip existing
        # cookies and attributes like response.csrf_processing_done
        response2 = HttpResponse(
            content=response._container,
            status=200,
            content_type=response['Content-Type'],
        )
        if response.has_header("Content-Language"):
            response2['Content-Language'] = response['Content-Language']

        if settings.DEBUG or RUN_WITH_DEV_SERVER or request.META.get('REMOTE_ADDR') in settings.INTERNAL_IPS:
            # Check if we store a {% csrf_token %} into the cache, this should never happen!
            for content in response._container:
                if "csrfmiddlewaretoken" in content:
                    raise AssertionError("csrf_token would be put into the cache! content: %r" % content)

        # Adds ETag, Last-Modified, Expires and Cache-Control headers
        patch_response_headers(response2, timeout)

        cache_key = get_cache_key(request)
        cache.set(cache_key, response2, timeout)

        logger.debug("Put to cache: %r" % cache_key)
        return response




"""
    :created: 2018 by Jens Diemer
    :copyleft: 2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

def print_mailbox(outbox):
    """
    Usefull for debugging tests, e.g.:

        # ... do something that send mails
        print_mailbox(mail.outbox)
        self.assertEqual(len(mail.outbox), 2)
    """
    for no, mail_instance in enumerate(outbox, start=1):
        print("_"*79)
        print(" *** Mail No. %i: ***" % no)
        for attr_name in dir(mail_instance):
            if attr_name.startswith("_"):
                continue
            attr = getattr(mail_instance, attr_name, "--")
            if callable(attr):
                continue
            if not isinstance(attr, (str, bytes, list, tuple, bool)) and attr is not None:
                continue
            if attr_name == "body":
                print("-"*79)
                print("%20s:" % attr_name)
                print(attr)
                print("-"*79)
            else:
                print("%20s: %s" % (attr_name, attr))
        print("="*79)



import tempfile
import unittest
from pathlib import Path

from django_tools.unittest_utils.isolated_filesystem import isolated_filesystem
from django_tools.unittest_utils.unittest_base import BaseUnittestCase


class TestIsolatedFilesystem(BaseUnittestCase):

    def setUp(self):
        super().setUp()
        self.old_cwd = Path().cwd()

        # set in self.assert_in_isolated_filesystem()
        self.temp_cwd = None
        self.temp_file = None

    def assert_in_isolated_filesystem(self, prefix=None):
        self.temp_cwd = Path().cwd()
        print(self.temp_cwd)
        self.assertNotEqual(self.temp_cwd, self.old_cwd)

        temp_dir = tempfile.gettempdir()
        self.assert_startswith(str(self.temp_cwd), temp_dir)

        if prefix is not None:
            reference = Path(temp_dir, prefix)
            self.assert_startswith(str(self.temp_cwd), str(reference))

        self.temp_file = Path(self.temp_cwd, "test_file.txt")

        with self.temp_file.open("w") as f:
            f.write(prefix or "no prefix")

        self.assertTrue(self.temp_file.is_file())

    def assert_after(self):
        self.assertTrue(self.temp_cwd is not None, "ERROR: 'self.assert_in_isolated_filesystem()' not called?!?")
        self.assertFalse(self.temp_cwd.is_dir(), "ERROR: %r still exists?!?" % self.temp_cwd)
        self.assertFalse(self.temp_file.is_file(), "ERROR: %r still exists!" % self.temp_file)

    def test_as_context_manager(self):
        prefix="test_as_context_manager"
        with isolated_filesystem(prefix=prefix):
            self.assert_in_isolated_filesystem(prefix)

        self.assert_after()

    def test_as_func_decotator(self):

        @isolated_filesystem()
        def in_isolated_filesystem():
            self.assert_in_isolated_filesystem(prefix="in_isolated_filesystem")

        in_isolated_filesystem()
        self.assert_after()

    def test_as_class_decotator(self):

        @isolated_filesystem()
        class InIsolatedFilesystem(unittest.TestCase):
            def call_assert_in_isolated_filesystem(self, parent):
                parent.assert_in_isolated_filesystem(prefix="InIsolatedFilesystem")

        test_case = InIsolatedFilesystem()
        test_case.setUp()
        test_case.call_assert_in_isolated_filesystem(parent=self)
        test_case.tearDown()

        self.assert_after()



import shutil
import tempfile


class TempDir():
    def __init__(self, prefix=""):
        self.tempfolder = tempfile.mkdtemp(prefix=prefix)

    def __enter__(self):
        return self.tempfolder

    def __exit__(self, exc_type, exc_value, traceback):
        shutil.rmtree(self.tempfolder)


# coding: utf-8

"""
    Test django_tools.unittest_utils.django_command
"""

from __future__ import print_function, unicode_literals

import os

import pytest

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from django.core.management import call_command
from django.test import TestCase

# https://github.com/jedie/django-tools
import django_tools
from django_tools.unittest_utils.django_command import DjangoCommandMixin
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer
from django_tools.unittest_utils.user import TestUserMixin

MANAGE_DIR = os.path.abspath(os.path.join(os.path.dirname(django_tools.__file__), ".."))


class TestListModelsCommand(DjangoCommandMixin, TestCase):
    def test_help(self):
        output = self.call_manage_py(["--help"], manage_dir=MANAGE_DIR)

        self.assertIn("[django]", output)
        self.assertIn("[django_tools]", output)
        self.assertIn("list_models", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_list_models(self):
        output = self.call_manage_py(["list_models"], manage_dir=MANAGE_DIR)
        print(output)

        self.assertIn("existing models in app_label.ModelName format:", output)

        self.assertIn("01 - admin.LogEntry", output)
        self.assertIn("02 - auth.Group", output)

        self.assertIn("06 - django_tools_test_app.LimitToUsergroupsTestModel", output)
        self.assertIn("07 - django_tools_test_app.PermissionTestModel", output)

        self.assertIn("INSTALLED_APPS....: 13", output)
        self.assertIn("Apps with models..: 13", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

class TestNiceDiffSettingsCommand(DjangoCommandMixin, TestCase):
    def test_help(self):
        output = self.call_manage_py(["--help"], manage_dir=MANAGE_DIR)

        self.assertIn("[django]", output)
        self.assertIn("[django_tools]", output)
        self.assertIn("nice_diffsettings", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_nice_diffsettings(self):
        output = self.call_manage_py(["nice_diffsettings"], manage_dir=MANAGE_DIR)
        print(output)

        self.assertIn("\n\nSETTINGS_MODULE = 'django_tools_test_project.test_settings'\n\n", output)
        self.assertIn("\n\nINSTALLED_APPS = ('django.contrib.auth',\n", output)

        self.assertNotIn("Traceback ", output) # Space after Traceback is important ;)
        self.assertNotIn("ERROR", output)


@pytest.mark.django_db
class TestPermissionInfoCommand(TestUserMixin, DjangoCommandMixin, TestCase):
    def test_environment(self):
        usernames = ",".join(
            self.UserModel.objects.values_list("username", flat=True).order_by("username")
        )
        self.assertEqual(usernames, "normal_test_user,staff_test_user,superuser")

    def test_help(self):
        output = self.call_manage_py(["--help"], manage_dir=MANAGE_DIR)

        self.assertIn("[django]", output)
        self.assertIn("permission_info", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_no_username_given(self):
        with StdoutStderrBuffer() as buff:
            call_command("permission_info")
        output = buff.get_output()
        print(output)

        self.assertIn("All existing users are:", output)
        self.assertIn("normal_test_user, staff_test_user, superuser", output)
        self.assertIn("(3 users)", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_wrong_username_given(self):
        with StdoutStderrBuffer() as buff:
            call_command("permission_info", "not existing username")
        output = buff.get_output()
        print(output)

        self.assertIn("Username 'not existing username' doesn't exists", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_normal_test_user(self):
        with StdoutStderrBuffer() as buff:
            call_command("permission_info", "normal_test_user")
        output = buff.get_output()
        print(output)

        self.assertIn("Display effective user permissions", output)
        self.assertIn("is_active    : yes", output)
        self.assertIn("is_staff     : no", output)
        self.assertIn("is_superuser : no", output)

        self.assertIn("[ ] auth.add_user", output)
        self.assertIn("[ ] sites.delete_site", output)

        self.assertNotIn("[*]", output) # normal user hasn't any permissions ;)
        self.assertNotIn("missing:", output) # no missing permissions

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)


class TestUpdatePermissionCommand(TestUserMixin, DjangoCommandMixin, TestCase):
    """
    Test for:
    django_tools.management.commands.update_permissions.Command
    """
    def test_help(self):
        output = self.call_manage_py(["--help"], manage_dir=MANAGE_DIR)

        self.assertIn("[django]", output)
        self.assertIn("update_permissions", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_update_permissions(self):
        with StdoutStderrBuffer() as buff:
            call_command("update_permissions")
        output = buff.get_output()
        print(output)

        self.assertIn("Create permissions for:", output)
        self.assertIn(" * auth", output)
        self.assertIn(" * django_tools_test_app", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)


class TestClearCacheCommand(DjangoCommandMixin, TestCase):
    """
    Test for:
    django_tools.management.commands.clear_cache.Command
    """
    def test_help(self):
        output = self.call_manage_py(["--help"], manage_dir=MANAGE_DIR)

        self.assertIn("[django]", output)
        self.assertIn("clear_cache", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)

    def test_clear_cache(self):

        cache.set("key_foo", "bar", 30)
        self.assertEqual(cache.get("key_foo"), "bar")

        with StdoutStderrBuffer() as buff:
            call_command("clear_cache")
        output = buff.get_output()
        print(output)

        self.assertEqual(cache.get("key_foo"), None)

        self.assertIn("Clear caches:", output)
        self.assertIn("Clear 'LocMemCache'", output)
        self.assertIn("done.", output)

        self.assertNotIn("Traceback", output)
        self.assertNotIn("ERROR", output)






"""
    :created: 06.04.2018 by Jens Diemer
    :copyleft: 2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import logging

# https://github.com/jedie/django-tools
from django_tools.unittest_utils.logging_utils import FilterAndLogWarnings, LoggingBuffer
from django_tools.unittest_utils.unittest_base import BaseUnittestCase

log = logging.getLogger(__name__)


class TestLoggingUtilsTestCase(BaseUnittestCase):
    def test_filter_and_log_warnings_create_warning(self):

        with LoggingBuffer(name=None, level=logging.DEBUG) as log_buffer:
            log.debug("A test log...")

        self.assertEqual(log_buffer.get_messages(), "DEBUG:django_tools_tests.test_logging_utils:A test log...")

    def test_filter_and_log_warnings_direct_call(self):
        instance = FilterAndLogWarnings()
        with LoggingBuffer(name=None, level=logging.DEBUG) as log_buffer:
            instance(
                message="test_filter_and_log_warnings_direct_call",
                category=UserWarning,
                filename="/foo/bar.py",
                lineno=123,
            )

        self.assertEqual(
            log_buffer.get_messages(),
            "WARNING:django_tools.unittest_utils.logging_utils:UserWarning:test_filter_and_log_warnings_direct_call",
        )

    def test_filter_and_log_skip_external_package(self):
        instance = FilterAndLogWarnings()
        with LoggingBuffer(name=None, level=logging.DEBUG) as log_buffer:
            for i in range(3):
                instance(
                    message="test_filter_and_log_skip_external_package dist-packages %i" % i,
                    category=UserWarning,
                    filename="/foo/dist-packages/bar.py",
                    lineno=456,
                )
                instance(
                    message="test_filter_and_log_skip_external_package site-packages %i" % i,
                    category=UserWarning,
                    filename="/foo/site-packages/bar.py",
                    lineno=789,
                )

        log_messages = log_buffer.get_messages()
        print("log_messages:\n%s" % log_messages)

        self.assertEqual(
            log_messages,
            (
                "WARNING:django_tools.unittest_utils.logging_utils:There are warnings in: /foo/dist-packages/bar.py\n"
                "WARNING:django_tools.unittest_utils.logging_utils:There are warnings in: /foo/site-packages/bar.py"
            ),
        )



# coding:utf-8

"""
    Dynamic SITE ID
    ~~~~~~~~~~~~~~~

    Set the SITE_ID dynamic by the current Domain Name.

    More info: read .../django_tools/dynamic_site/README.creole

    :copyleft: 2011-2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import logging
import os
import sys
import warnings

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

from django.conf import settings
from django.contrib.sites import models as sites_models
from django.core.exceptions import MiddlewareNotUsed, ImproperlyConfigured

from django_tools.local_sync_cache.local_sync_cache import LocalSyncCache

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object  # fallback for Django < 1.10


USE_DYNAMIC_SITE_MIDDLEWARE = getattr(settings, "USE_DYNAMIC_SITE_MIDDLEWARE", False)

logger = logging.getLogger("django_tools.DynamicSite")


Site = sites_models.Site  # Shortcut


class DynamicSiteId(object):
    def __getattribute__(self, name):
#        print "DynamicSiteId __getattribute__", name
        return getattr(SITE_THREAD_LOCAL.SITE_ID, name)
    def __int__(self):
#        print "DynamicSiteId __int__"
        return SITE_THREAD_LOCAL.SITE_ID
    def __hash__(self):
#        print "DynamicSiteId __hash__"
        return hash(SITE_THREAD_LOCAL.SITE_ID)
    def __repr__(self):
#        print "DynamicSiteId __repr__"
        return repr(SITE_THREAD_LOCAL.SITE_ID)
    def __str__(self):
#        print "DynamicSiteId __str__"
        return str(SITE_THREAD_LOCAL.SITE_ID)
    def __unicode__(self):
#        print "DynamicSiteId __unicode__"
        return str(SITE_THREAD_LOCAL.SITE_ID)


def _clear_cache(self):
    logger.debug("Clear SITE_CACHE (The django-tools LocalSyncCache() dict)")
    SITE_CACHE.clear()


if USE_DYNAMIC_SITE_MIDDLEWARE == True:
    # Use the same SITE_CACHE for getting site object by host [1] and get current site by SITE_ID [2]
    # [1] here in DynamicSiteMiddleware._get_site_id_from_host()
    # [2] in django.contrib.sites.models.SiteManager.get_current()
    SITE_CACHE = LocalSyncCache(id="DynamicSiteMiddlewareCache")
    sites_models.SITE_CACHE = SITE_CACHE

    SITE_THREAD_LOCAL = local()

    # Use Fallback ID if host not exist in Site table. We use int() here, because
    # os environment variables are always strings.
    FALLBACK_SITE_ID = int(getattr(os.environ, "SITE_ID", settings.SITE_ID))
    logger.debug("Fallback SITE_ID: %r" % FALLBACK_SITE_ID)

    # Use Fallback ID at startup before process_request(), e.g. in unittests
    SITE_THREAD_LOCAL.SITE_ID = FALLBACK_SITE_ID

    try:
        FALLBACK_SITE = Site.objects.get(id=FALLBACK_SITE_ID)
    except Site.DoesNotExist as e:
        all_sites = Site.objects.all()
        msg = "Fallback SITE_ID %i doesn't exist: %s (Existing sites: %s)" % (
            FALLBACK_SITE_ID, e, repr(all_sites.values_list("id", "domain").order_by('id'))
        )
        logger.critical(msg)
        raise ImproperlyConfigured(msg)
#        site = Site(id=FALLBACK_SITE_ID, domain="example.tld", name="Auto Created!")
#        site.save()

    settings.SITE_ID = DynamicSiteId()

    # Use the same cache for Site.objects.get_current():
    sites_models.SITE_CACHE = SITE_CACHE

    # monkey patch for django.contrib.sites.models.SiteManager.clear_cache
    sites_models.SiteManager.clear_cache = _clear_cache


class DynamicSiteMiddleware(MiddlewareMixin):
    """ Set settings.SITE_ID based on request's domain. """

    def __init__(self):
        # User must add "USE_DYNAMIC_SITE_MIDDLEWARE = True" in his local_settings.py
        # to activate this middleware
        if USE_DYNAMIC_SITE_MIDDLEWARE != True:
            logger.info("DynamicSiteMiddleware is deactivated.")
            raise MiddlewareNotUsed()
        else:
            logger.info("DynamicSiteMiddleware is active.")

    def process_request(self, request):
        # Get django.contrib.sites.models.Site instance by the current domain name:
        site = self._get_site_id_from_host(request)

        # Save the current site
        SITE_THREAD_LOCAL.SITE_ID = site.pk

        # Put site in cache for django.contrib.sites.models.SiteManager.get_current():
        SITE_CACHE[SITE_THREAD_LOCAL.SITE_ID] = site

#        def test():
#            from django.contrib.sites.models import Site, SITE_CACHE
#            from django.conf import settings
#            print id(SITE_CACHE), SITE_CACHE
#            print "-"*79
#            for k, v in SITE_CACHE.items():
#                print k, type(k), id(k), hash(k), v
#            print "-"*79
#            print id(settings.SITE_ID), settings.SITE_ID
#            print "TEST:", Site.objects.get_current()
#        test()

    def _get_site_id_from_host(self, request):
        host = request.get_host().lower()
        try:
            return SITE_CACHE[host]
        except KeyError:
            site = self._get_site_from_host(host)
            if site is None:
                # Fallback:
                logger.critical("Use FALLBACK_SITE !")
                site = FALLBACK_SITE
            else:
                logger.debug("Set site to %r for %r" % (site, host))
                SITE_CACHE[host] = site
            return site

    def _get_site_from_host(self, host):
        site = None
        try:
            site = Site.objects.get(domain__iexact=host)
        except Site.DoesNotExist:
            # Look if there is a alias
            from django_tools.dynamic_site.models import SiteAlias # against import loops ;(
            try:
                site = SiteAlias.objects.get_from_host(host)
            except SiteAlias.DoesNotExist:
                # FIXME: How can we give better feedback?
                all_sites = Site.objects.all()
                msg = "Error: There exist no SITE entry for domain %r! (Existing domains: %s)" % (
                    host, repr(all_sites.values_list("id", "domain").order_by('id'))
                )
                logger.critical(msg)
#                if settings.DEBUG:
#                    raise RuntimeError(msg)
#                else:
                warnings.warn(msg)
        return site




# coding: utf-8

"""
    Send warnings if a template variable does not exist in the content.

    experimental.

    Put this into your settings.py:
    --------------------------------------------------------------------------
    if DEBUG:
        from django_tools.template import warn_invalid_template_vars
        warn_invalid_template_vars.add_warning()
    --------------------------------------------------------------------------
"""

import warnings

from django import template
# from django.utils.encoding import smart_unicode, force_unicode, smart_str

_WARN_ADDED = False

MAX_LEN = 79

class InvalidTemplateKey(Warning):
    pass

def add_warning():
    global _WARN_ADDED
    if _WARN_ADDED:
        return

    class WarnVariableDoesNotExist(template.VariableDoesNotExist):
        def __init__(self, *args, **kwargs):
            super(WarnVariableDoesNotExist, self).__init__(*args, **kwargs)

            warn_msg = str(self) # get the complete message encoded in UTF-8
            if len(warn_msg) > MAX_LEN:
                warn_msg = warn_msg[:MAX_LEN] + "..."
            warnings.warn(warn_msg, category=InvalidTemplateKey)

    template.VariableDoesNotExist = WarnVariableDoesNotExist

    _WARN_ADDED = True





import logging

from django.conf import settings
from django.utils import translation


log = logging.getLogger(__name__)


def iter_languages(languages=None):
    """
    Iterate over all existing languages with activated translations.
    """
    if languages is None:
        languages = settings.LANGUAGES

    for language_code, lang_name in languages:
        with translation.override(language_code):
            yield language_code, lang_name



# coding: utf-8

"""
    database_info manage command


    setup:

        INSTALLED_APPS = [
            ...
            'django_tools',
            ...
        ]


    usage:

        $ ./manage.py database_info


    :copyleft: 2017 by the django-tools team, see AUTHORS for more details.
    :created: 2017 by Jens Diemer
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from pprint import pprint

from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import connections


class Command(BaseCommand):
    help = "Information about the used database and connections"

    def handle(self, *args, **options):
        self.stdout.write("")
        self.stdout.write("_"*79)
        self.stdout.write(self.help)
        self.stdout.write("")

        # self.stdout.write("settings.DATABASES = ", ending="")
        # pprint(settings.DATABASES)
        # self.stdout.write("-"*79)

        for alias, settings_dict in settings.DATABASES.items():
            self.stdout.write("Database alias %r:" % alias)
            self.stdout.write("")
            engine = settings_dict["ENGINE"]
            engine = engine.rsplit(".",1)[-1]
            self.stdout.write("engine...............: %r" % engine)
            if engine == "sqlite3":
                # https://docs.python.org/3/library/sqlite3.html#module-functions-and-constants
                import sqlite3
                self.stdout.write("sqlite lib version...: %r" % sqlite3.sqlite_version)
                self.stdout.write("sqlite module version: %r" % sqlite3.version)

            self.stdout.write("name.................: %r" % settings_dict["NAME"])
            self.stdout.write("user.................: %r" % settings_dict["USER"])
            self.stdout.write("host.................: %r" % settings_dict["HOST"])
            self.stdout.write("port.................: %r" % settings_dict["PORT"])

        connection_list = connections.all()

        self.stdout.write("")
        self.stdout.write("There are %i connections." % len(connection_list))

        for no, conn in enumerate(connection_list, 1):
            self.stdout.write("")
            self.stdout.write("connection %i alias: %r settings_dict:" % (no, conn.alias))
            pprint(conn.settings_dict)

        self.stdout.write("")
        self.stdout.write("-"*79)



# coding: utf-8

"""
    Auto update Filebased cache
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import logging
import time
import sys
import os
try:
    import pickle as pickle
except ImportError:
    import pickle

from django.core.cache.backends.filebased import FileBasedCache
from django.conf import settings


AUTOUPDATECACHE_CHANGE_TIME = getattr(settings, "AUTOUPDATECACHE_CHANGE_TIME", "AUTOUPDATECACHE_CHANGE_TIME")
AUTOUPDATECACHE_UPDATE_TIMESTAMP = getattr(settings, "AUTOUPDATECACHE_UPDATE_TIMESTAMP", 10)
AUTOUPDATECACHE_TIMES = getattr(settings, "AUTOUPDATECACHE_TIMES", (
    # load value, max age in sec.
    (0, 10), #     < 0.5     -> 10sec
    (0.5, 30), #   0.5 - 1.0 -> 30sec
    (1.0, 60), #   1.0 - 1.5 ->  1Min
    (1.5, 120), #  1.5 - 2.0 ->  2Min
    (2.0, 300), #  2.0 - 3.0 ->  5Min
    (3.0, 900), #  3.0 - 4.0 -> 15Min
    (4.0, 3600), # > 4.0     ->  1h
))

# Sort from biggest to lowest:
AUTOUPDATECACHE_TIMES = list(AUTOUPDATECACHE_TIMES)
AUTOUPDATECACHE_TIMES.sort(reverse=True)
AUTOUPDATECACHE_TIMES = tuple(AUTOUPDATECACHE_TIMES)

def get_max_age(load_average):
    """ return max age for the given load average. """
    load_average += 0.1
    for load, max_age in AUTOUPDATECACHE_TIMES:
        if load_average > load:
            break
    return max_age


logger = logging.getLogger("SmoothyFileBasedCache")

#if "runserver" in sys.argv or "tests" in sys.argv:
#    log.logging.basicConfig(format='%(created)f pid:%(process)d %(message)s')
#    logger.setLevel(log.logging.DEBUG)
#    logger.addHandler(log.logging.StreamHandler())
#
#if not logger.handlers:
#    # ensures we don't get any 'No handlers could be found...' messages
#    logger.addHandler(log.NullHandler())


class AutoUpdateFileBasedCache(FileBasedCache):
    CHANGE_TIME = None # Timestamp of the "last update"
    NEXT_SYNC = None # Point in the future to update the CHANGE_TIME

    def __init__(self, *args, **kwargs):
        super(AutoUpdateFileBasedCache, self).__init__(*args, **kwargs)
        import warnings
        warnings.warn(
            "django-tools AutoUpdateFileBasedCache is deprecated, use new SmoothCacheBackends!",
            category=DeprecationWarning
        )

    def save_change_time(self):
        """
        save the "last change" timestamp to renew the cache entries in
        self.CHANGE_TIME and in cache.
        """
        now = time.time()
        self.CHANGE_TIME = now
        self.set(AUTOUPDATECACHE_CHANGE_TIME, now)
        logger.debug("Set CHANGE_TIME to %r" % now)

    def get_change_time(self):
        """
        return current "last change" timestamp.
        To save cache access, update the timestamp only in  
        AUTOUPDATECACHE_UPDATE_TIMESTAMP frequency from cache.
        """
        now = time.time()
        if now > self.NEXT_SYNC or self.CHANGE_TIME is None:
            # Update timestamp we must look for a new change time from cache:
            self.NEXT_SYNC = now + AUTOUPDATECACHE_UPDATE_TIMESTAMP

            change_time = super(AutoUpdateFileBasedCache, self).get(AUTOUPDATECACHE_CHANGE_TIME)
            if change_time is None:
                logger.debug("CHANGE_TIME is None")
                self.save_change_time() # save change time into cache             
            elif change_time > self.CHANGE_TIME:
                self.CHANGE_TIME = change_time
                logger.debug("update change time to: %r" % change_time)
        else:
            logger.debug("Use old CHANGE_TIME %r" % self.CHANGE_TIME)

        return self.CHANGE_TIME

    def must_updated(self, key, create_time):
        """
        return True if given cache create time is older than the "last change"
        time, but only if the additional time from system load allows it.
        """
        last_change_time = self.get_change_time()
        if last_change_time < create_time:
            logger.debug("Cache item %r not out-dated" % key)
            return False

        outdate_age = last_change_time - create_time
        load_average = os.getloadavg()[0] # load over last minute
        max_age = get_max_age(load_average)
        if outdate_age > max_age:
            logger.debug("Out-dated %r (age: %s, max age: %s, load: %s)" % (
                key, outdate_age, max_age, load_average
            ))
            return True

        logger.debug("Keep %r by load (out-dated age: %s, max age: %s, load: %s)" % (
            key, outdate_age, max_age, load_average
        ))
        return False

    def get(self, key, default=None, version=None):
        key = self.make_key(key, version=version)
        self.validate_key(key)

        fname = self._key_to_file(key)
        try:
            f = open(fname, 'rb')
            try:
                exp = pickle.load(f)
                now = time.time()
                if exp < now:
                    self._delete(fname)
                else:
                    #----------------------------------------------------------
                    # START area of changes to the original get() method.
                    #
                    # Use the modify time of the cache file and
                    # compare it with the "last change" time. 
                    mtime = os.fstat(f.fileno()).st_mtime
                    if self.must_updated(key, mtime):
                        self._delete(fname)
                    else:
                        return pickle.load(f)

                    # END area of changes to the original function.
                    #----------------------------------------------------------
            finally:
                f.close()
        except (IOError, OSError, EOFError, pickle.PickleError):
            pass
        return default




# coding: utf-8

"""
    Dynamic SITE ID - model admin
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


from django.conf import settings

from django.contrib import admin
from django_tools.dynamic_site.models import SiteAlias


class SiteAliasAdmin(admin.ModelAdmin):
    list_display = (
        "id", "site", "alias", "regex"
    )
    list_display_links = ("id", "alias")
    list_filter = ("site", "regex")
    search_fields = ("alias",)


if getattr(settings, "USE_DYNAMIC_SITE_MIDDLEWARE", False) == True:
    admin.site.register(SiteAlias, SiteAliasAdmin)



# coding: utf-8

"""
    'clear django cache' manage command


    setup:

        INSTALLED_APPS = [
            ...
            'django_tools',
            ...
        ]


    usage:

        $ ./manage.py clear_cache


    :copyleft: 2017 by the django-tools team, see AUTHORS for more details.
    :created: 2017 by Jens Diemer
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.core.cache import caches
from django.core.management.base import BaseCommand

class Command(BaseCommand):
    help = "Clears the complete Django cache."

    def handle(self, *args, **options):
        self.stdout.write("\nClear caches:\n")
        for cache in caches.all():
            self.stdout.write("\tClear '%s'\n" % cache.__class__.__name__)
            cache.clear()

        self.stdout.write("\ndone.\n")



"""
    :created: 24.04.2018 by Jens Diemer
    :copyleft: 2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.apps import apps
from django.db import models


class ModelTestGenerator:

    def _iter_model_label(self):
        app_configs = apps.get_app_configs()

        data = []
        for app_config in app_configs:
            models = app_config.get_models()
            for model in models:
                data.append((model._meta.label, model))

        yield from sorted(data)

    def get_model_by_label(self, model_label):
        for label, model in self._iter_model_label():
            if label == model_label:
                return model

    def get_models_startwith_label(self, prefix):
        models = []
        for label, model in self._iter_model_label():
            if label.startswith(prefix):
                models.append(model)

        return models

    def print_all_plugins(self):
        print("All models:")
        app_configs = apps.get_app_configs()
        for app_config in app_configs:
            models = app_config.get_models()
            for model in models:
                qs = model.objects.all()
                print("%6i instances: %r" % (qs.count(), model._meta.label))

    def test_code_for_instance(self, instance):
        prefix_lines = []
        lines = [
            "#", "# pk:{pk} from {label} {type}".format(
                pk=instance.pk,
                label=instance._meta.label,
                type=type(instance),
            ), "#", "{name} = {ClassName}.objects.create(".format(
                name=instance._meta.model_name,
                ClassName=instance._meta.object_name,
            )
        ]
        for field in instance._meta.fields:
            # field == django.db.models.fields.Field

            if field.hidden or field.auto_created:
                continue

            comment_post_line = False

            if not field.editable:
                comment_post_line = True

            internal_type = field.get_internal_type()

            value = getattr(instance, field.name)
            if isinstance(value, models.Model):
                prefix_lines += self.test_code_for_instance(instance=value)
                lines.append(
                    "    {name}={obj_name}.pk, # {internal_type} to {label}".format(
                        name=field.name,
                        obj_name=value._meta.model_name,
                        internal_type=internal_type,
                        label=value._meta.label
                    )
                )
                continue

            comment = field.description % {
                "max_length": field.max_length,
            }

            line = "{name}={value!r}, # {internal_type}, {comment}".format(
                name=field.name, value=value, internal_type=internal_type, comment=comment
            )
            if comment_post_line:
                line = "# %s" % line

            lines.append("    %s" % line)

        lines.append(")")
        lines.append("")

        if prefix_lines:
            lines = prefix_lines + lines

        return lines

    def from_queryset(self, queryset):
        lines = []
        for instance in queryset:
            lines += self.test_code_for_instance(instance)
            lines.append("#" * 79)

        content = "\n".join(lines)
        return content



"""
    :created: 2015 by Jens Diemer
    :copyleft: 2015-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import logging
import pprint
import sys
import traceback
import unittest

from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

# https://github.com/jedie/django-tools
from django_tools.selenium.response import selenium2fakes_response

log = logging.getLogger(__name__)


class LocalStorage:
    """
    https://developer.mozilla.org/de/docs/Web/API/Window/localStorage

    Note:
        * Access "window.localStorage" works only *after* a request.
        * There's no type conversion!

    Otherwise it ends in:

    selenium.common.exceptions.WebDriverException:
        Message: move target out of bounds:
            Failed to read the 'localStorage' property from 'Window': Storage is disabled inside 'data:' URLs.
    """

    def __init__(self, driver):
        self.driver = driver

    def items(self):
        return self.driver.execute_script(
            "var ls = window.localStorage, items = {}; "
            "for (var i = 0, k; i < ls.length; ++i) "
            "  items[k = ls.key(i)] = ls.getItem(k); "
            "return items; "
        )

    def keys(self):
        return self.driver.execute_script(
            "var ls = window.localStorage, keys = []; "
            "for (var i = 0; i < ls.length; ++i) "
            "  keys[i] = ls.key(i); "
            "return keys; "
        )

    def clear(self):
        self.driver.execute_script("window.localStorage.clear();")

    def __len__(self):
        return self.driver.execute_script("return window.localStorage.length;")

    def __getitem__(self, key):
        value = self.driver.execute_script("return window.localStorage.getItem(arguments[0]);", key)
        if value is None:
            raise KeyError(key)
        return value

    def __setitem__(self, key, value):
        self.driver.execute_script("window.localStorage.setItem(arguments[0], arguments[1]);", key, value)

    def __contains__(self, key):
        return key in self.keys()

    def __repr__(self):
        return self.items().__str__()


class SeleniumBaseTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        if cls.driver is not None:
            cls.local_storage = LocalStorage(cls.driver)

    @classmethod
    def tearDownClass(cls):
        if cls.driver is not None:
            cls.driver.quit()
        super().tearDownClass()

    def setUp(self):
        super().setUp()
        if self.driver is not None:
            self.driver.delete_all_cookies()

    def tearDown(self):
        super().tearDown()
        if self.driver is not None:
            # we clear window.localStorage here and not in setUp(), because
            # access "window.localStorage" works only *after* a request
            self.local_storage.clear()

    def _wait(self, conditions, timeout=5, msg="wait timeout"):
        """
        Wait for the given condition.
        Display page_source on error.
        """
        try:
            check = WebDriverWait(self.driver, timeout).until(conditions)
        except TimeoutException as err:
            print("\nError: %s\n%s\npage source:\n%s\n" % (msg, err, self.driver.page_source))
            raise
        else:
            self.assertTrue(check)

    def get_faked_response(self):
        """
        Create a similar 'testing-response' [1] here.
        So that some of the django testing assertions [2] can be used
        with selenium tests, too ;)

        Currently not available:
            * response.status_code
            * response.redirect_chain
            * response.templates
            * response.context

        Available:
            * response.content
            * response.cookies
            * response.client.cookies
            * response.session

        [1] https://docs.djangoproject.com/en/1.11/topics/testing/tools/#testing-responses
        [2] https://docs.djangoproject.com/en/1.11/topics/testing/tools/#assertions
        """
        return selenium2fakes_response(self.driver, self.client, self.client_class)

    def _verbose_assertion_error(self, err):
        sys.stderr.write("\n\n")
        sys.stderr.flush()
        sys.stderr.write("*" * 79)
        sys.stderr.write("\n")

        traceback.print_exc()

        sys.stderr.write(" -" * 40)
        sys.stderr.write("\n")
        try:
            page_source = self.driver.page_source
        except Exception as e:
            print("Can't get 'driver.page_source': %s" % e)
        else:
            page_source = "\n".join([line for line in page_source.splitlines() if line.rstrip()])
            print(page_source, file=sys.stderr)

        sys.stderr.write("*" * 79)
        sys.stderr.write("\n")
        sys.stderr.write("\n\n")
        sys.stderr.flush()

        raise AssertionError(err)

    def assert_no_javascript_alert(self):
        alert = expected_conditions.alert_is_present()(self.driver)
        if alert != False:
            alert_text = alert.text
            alert.accept()  # Confirm a alert dialog, otherwise access to driver.page_source will failed!
            try:
                raise self.failureException("Alert is preset: %s" % alert_text)
            except AssertionError as err:
                self._verbose_assertion_error(err)

    def assert_equal_page_title(self, should):
        try:
            self.assertEqual(self.driver.title, should)
        except AssertionError as err:
            self._verbose_assertion_error(err)

    def assert_in_page_source(self, member):
        try:
            self.assertIn(member, self.driver.page_source)
        except AssertionError as err:
            self._verbose_assertion_error(err)

    def assert_not_in_page_source(self, member):
        try:
            self.assertNotIn(member, self.driver.page_source)
        except AssertionError as err:
            self._verbose_assertion_error(err)

    def assert_visible_by_id(self, id, timeout=10):
        """
        Test that an element by ID is present on the DOM of a page and visible.
        """
        locator = (By.ID, id)
        self._wait(
            conditions=expected_conditions.visibility_of_element_located(locator),
            timeout=timeout,
            msg="Wait for '#%s' to be visible. (timeout: %i)" % (id, timeout),
        )

    def assert_clickable_by_id(self, id, timeout=10):
        """
        Test that an element by ID is visible and enabled such that you can click it
        """
        locator = (By.ID, id)
        self._wait(
            conditions=expected_conditions.element_to_be_clickable(locator),
            timeout=timeout,
            msg="Wait for '#%s' to be clickable. (timeout: %i)" % (id, timeout),
        )

    def assert_clickable_by_xpath(self, xpath, timeout=10):
        """
        Test that an element by ID is visible and enabled such that you can click it
        """
        locator = (By.XPATH, xpath)
        self._wait(
            conditions=expected_conditions.element_to_be_clickable(locator),
            timeout=timeout,
            msg="Wait for '%s' to be clickable. (timeout: %i)" % (xpath, timeout),
        )

    def assert_local_storage_key_value(self, *, key, value):
        print("local_storage:", pprint.pformat(self.local_storage))
        is_value = self.local_storage[key]
        self.assertEqual(value, is_value)



"""
    :copyleft: 2017-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys

import django
from django.core import mail
from django.test import Client, SimpleTestCase
from django.utils import six
from django.utils.six import PY2

from django_tools_test_project.django_tools_test_app.models import PermissionTestModel

# https://github.com/jedie/django-tools
from django_tools.mail.send_mail import SendMail
from django_tools.template.render import render_string_template
from django_tools.unittest_utils.email import print_mailbox
from django_tools.unittest_utils.print_sql import PrintQueries
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer
from django_tools.unittest_utils.tempdir import TempDir
from django_tools.unittest_utils.template import TEMPLATE_INVALID_PREFIX, set_string_if_invalid
from django_tools.unittest_utils.unittest_base import BaseTestCase, BaseUnittestCase
from django_tools.unittest_utils.user import TestUserMixin


class TestBaseUnittestCase(BaseUnittestCase):

    def test_dedent(self):
        """
        Test for BaseUnittestCase._dedent()
        """
        out1 = self._dedent("""
            one line
            line two""")
        self.assertEqual(out1, "one line\nline two")

        out2 = self._dedent("""
            one line
            line two
        """)
        self.assertEqual(out2, "one line\nline two")

        out3 = self._dedent("""
            one line

            line two
        """)
        self.assertEqual(out3, "one line\n\nline two")

        out4 = self._dedent("""
            one line
                line two

        """)
        self.assertEqual(out4, "one line\n    line two")

        # removing whitespace and the end
        self.assertEqual(self._dedent("\n  111  \n  222"), "111\n222")

        out5 = self._dedent("""
            one line
                line two
            dritte Zeile
        """)
        self.assertEqual(out5, "one line\n    line two\ndritte Zeile")

    def test_assertEqual_dedent(self):
        self.assertEqual_dedent(first="foo bar", second="foo bar")

        with self.assertRaises(AssertionError) as cm:
            self.assertEqual_dedent(first="foo bar", second="foo X bar")

        err_msg = "\n".join([line.strip() for line in cm.exception.args[0].splitlines()])
        if PY2:
            err_msg = err_msg.replace("u'", "'")
        print("***\n%s\n***" % err_msg)
        self.assertEqual(
            err_msg,
            self._dedent(
                """
            'foo bar' != 'foo X bar'
            - foo bar
            + foo X bar
            ?    ++
        """
            )
        )

    def test_assertIn_dedent(self):
        self.assertIn_dedent(member="foo", container="The foo is here!")

        with self.assertRaises(AssertionError) as cm:
            self.assertIn_dedent(member="foo", container="only bar")

        err_msg = "\n".join([line.strip() for line in cm.exception.args[0].splitlines()])
        if PY2:
            err_msg = err_msg.replace("u'", "'")
        print("***\n%s\n***" % err_msg)
        self.assertEqual(err_msg, self._dedent("""
            'foo' not found in 'only bar'
        """))

    # TODO: remove assertments tests in the future!

    def test_assert_is_dir(self):
        existing_path = os.path.dirname(__file__)
        self.assert_is_dir(existing_path)
        self.assertRaises(AssertionError, self.assert_is_dir, "foobar_dir")

    def test_assert_not_is_dir(self):
        self.assert_not_is_dir("foobar_dir")
        existing_path = os.path.dirname(__file__)
        self.assertRaises(AssertionError, self.assert_not_is_dir, existing_path)

    def test_assert_is_file(self):
        self.assert_is_file(__file__)
        self.assertRaises(AssertionError, self.assert_is_file, "foobar_file.txt")

    def test_assert_not_is_File(self):
        self.assert_not_is_File("foobar_file.txt")
        self.assertRaises(AssertionError, self.assert_not_is_File, __file__)

    def test_assert_startswith(self):
        self.assert_startswith("foobar", "foo")
        self.assertRaises(AssertionError, self.assert_startswith, "foobar", "bar")

    def test_assert_endswith(self):
        self.assert_endswith("foobar", "bar")
        self.assertRaises(AssertionError, self.assert_endswith, "foobar", "foo")

    def test_get_admin_change_url(self):
        obj = PermissionTestModel.objects.create()
        url = self.get_admin_change_url(obj)
        if django.VERSION < (1, 11):
            self.assertEqual(url, "/admin/django_tools_test_app/permissiontestmodel/%i/" % obj.pk)
        else:
            self.assertEqual(url, "/admin/django_tools_test_app/permissiontestmodel/%i/change/" % obj.pk)

    def test_get_admin_add_url(self):
        url = self.get_admin_add_url(obj=PermissionTestModel)
        self.assertEqual(url, "/admin/django_tools_test_app/permissiontestmodel/add/")


class TestTempDir(BaseUnittestCase):

    def testTempDir(self):
        """ test TempDir() """
        with TempDir(prefix="foo_") as tempfolder:
            self.assert_is_dir(tempfolder)
            self.assertIn(os.sep + "foo_", tempfolder)

            test_filepath = os.path.join(tempfolder, "bar")
            open(test_filepath, "w").close()
            self.assert_is_file(test_filepath)

        # cleaned?
        self.assert_not_is_dir(tempfolder)
        self.assert_not_is_File(test_filepath)


class TestStdoutStderrBuffer(BaseUnittestCase):

    def test_text_type(self):
        with StdoutStderrBuffer() as buffer:
            print(six.text_type("print text_type"))
            sys.stdout.write(six.text_type("stdout.write text_type\n"))
            sys.stderr.write(six.text_type("stderr.write text_type"))
        self.assertEqual_dedent(
            buffer.get_output(), """
            print text_type
            stdout.write text_type
            stderr.write text_type
        """
        )

    def test_binary_type(self):
        if six.PY2:
            with StdoutStderrBuffer() as buffer:
                print("print str")
                sys.stdout.write("stdout.write str\n")
                sys.stderr.write("stderr.write str")
            self.assertEqual_dedent(
                buffer.get_output(), """
                print str
                stdout.write str
                stderr.write str
            """
            )
        elif six.PY3:
            # The print function will use repr
            with StdoutStderrBuffer() as buffer:
                print(b"print binary_type")
                sys.stdout.write(b"stdout.write binary_type\n")
                sys.stderr.write(b"stderr.write binary_type")
            self.assertEqual_dedent(
                buffer.get_output(), """
                b'print binary_type'
                stdout.write binary_type
                stderr.write binary_type
            """
            )
        else:
            self.fail()


class TestBaseTestCase(TestUserMixin, BaseTestCase):

    def test_print_sql(self):
        with StdoutStderrBuffer() as buffer:
            with PrintQueries("Create object"):
                self.UserModel.objects.all().count()

        output = buffer.get_output()
        # print(output)

        self.assertIn("*** Create object ***", output)
        # FIXME: Will fail if not SQLite/MySQL is used?!?
        if django.VERSION < (1, 9):
            self.assertIn("1 - QUERY = 'SELECT COUNT(", output)
        else:
            self.assertIn("1 - SELECT COUNT(", output)
        self.assertIn('FROM "auth_user"', output)

    def test_create_users(self):
        self.UserModel.objects.all().delete()

        self.create_testusers()

        usernames = self.UserModel.objects.all().values_list("username", flat=True)
        usernames = list(usernames)
        usernames.sort()
        self.assertEqual(usernames, ['normal_test_user', 'staff_test_user', 'superuser'])

        # Are all users active?
        self.assertEqual(self.UserModel.objects.filter(is_active=True).count(), 3)


@set_string_if_invalid()
class TestSetStringIfInvalidDecorator(SimpleTestCase):

    def assert_activation(self):
        from django.conf import settings
        string_if_invalid = settings.TEMPLATES[0]['OPTIONS']['string_if_invalid']
        self.assertIn(TEMPLATE_INVALID_PREFIX, string_if_invalid)

    def test_valid_tag(self):
        self.assert_activation()
        content = render_string_template("pre{{ tag }}post", {"tag": "FooBar!"})
        self.assertEqual(content, 'preFooBar!post')
        self.assertNotIn(TEMPLATE_INVALID_PREFIX, content)

    def test_invalid_tag(self):
        self.assert_activation()
        content = render_string_template("pre{{ tag }}post", {})
        self.assertEqual(content, 'pre***invalid:tag***post')
        self.assertIn(TEMPLATE_INVALID_PREFIX, content)


class AssertResponseTest(BaseTestCase):

    @classmethod
    def setUpClass(cls):
        super(AssertResponseTest, cls).setUpClass()
        cls.response = Client().get("/admin/login/")

    def test_assert_response_ok(self):
        self.assertResponse(
            self.response,
            must_contain=(
                "Django administration",
                "Username:",
                "Password:",
                'value="Log in"',
            ),
            must_not_contain=(
                "error",
                "traceback",
            ),
            status_code=200,
            template_name="admin/login.html",
            html=False,
            browser_traceback=True
        )

    def test_must_contain(self):
        self.assertResponse(self.response, must_contain=("Django administration",))
        self.assertRaises(
            AssertionError,
            self.assertResponse,
            self.response,
            must_contain=("Django X administration",),
            browser_traceback=False
        )

    def test_must_not_contain(self):
        self.assertRaises(
            AssertionError,
            self.assertResponse,
            self.response,
            must_not_contain=("Django administration",),
            browser_traceback=False
        )

    def test_wrong_template(self):
        self.assertResponse(self.response, template_name="admin/login.html")
        self.assertRaises(
            AssertionError,
            self.assertResponse,
            self.response,
            template_name="admin/loginXXX.html",
            browser_traceback=False
        )

    def test_wrong_status_code(self):
        self.assertResponse(self.response, status_code=200)
        self.assertRaises(AssertionError, self.assertResponse, self.response, status_code=404, browser_traceback=False)

    def test_get_messages_normal_response(self):
        response = self.client.get("/create_message_normal_response/normal-response-message/")

        self.assertEqual(self.get_messages(response), ['normal-response-message'])

        self.assertMessages(response, ['normal-response-message'])

        self.assertResponse(
            response,
            must_contain=("django_tools_test_app.views.create_message_normal_response",),
            must_not_contain=("error", "traceback"),
            status_code=200,
            messages=['normal-response-message'],
            html=False,
            browser_traceback=True
        )

    def test_get_messages_without_context(self):
        response = self.client.get("/create_message_redirect_response/redirect-response-message/")

        self.assertEqual(self.get_messages(response), ['redirect-response-message'])

        self.assertMessages(response, ['redirect-response-message'])

        self.assertResponse(
            response,
            status_code=302,
            messages=['redirect-response-message'],
        )

    def test_wrong_messages(self):
        response = self.client.get("/create_message_normal_response/this-is-it/")
        self.assertMessages(response, ['this-is-it'])

        with self.assertRaises(AssertionError) as cm:
            self.assertResponse(
                response,
                messages=['this-is-not-it'],
                browser_traceback=False,
            )

        self.assert_exception_startswith(cm, "Lists differ: ['this-is-it'] != ['this-is-not-it']")

    def test_print_mailbox(self):
        self.assertEqual(len(mail.outbox), 0)

        ok = SendMail(
            template_base="mail_test.{ext}",
            mail_context={
                "foo": "first",
                "bar": "second"
            },
            subject="test test_print_mailbox()",
            recipient_list="foo@bar.tld"
        ).send()

        self.assertEqual(ok, True)

        self.assertEqual(len(mail.outbox), 1)

        with StdoutStderrBuffer() as buff:
            print_mailbox(mail.outbox)
        output = buff.get_output()
        print(output)

        self.assertIn("*** Mail No. 1: ***", output)
        self.assertIn("subject: test test_print_mailbox()", output)
        self.assertIn("<!-- START 'mail_test.txt' -->", output)
        self.assertIn("<!-- END 'mail_test.txt' -->", output)
        self.assertIn("from_email: webmaster@localhost", output)
        self.assertIn("to: ['foo@bar.tld']", output)



"""
    :created: 11.12.2018 by Jens Diemer
    :copyleft: 2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import unittest

# https://github.com/jedie/django-tools
from django_tools.unittest_utils.selenium_utils import (
    SeleniumChromiumTestCase, SeleniumFirefoxTestCase, chromium_available, find_executable, firefox_available
)


class TestWarnDecorators(unittest.TestCase):
    def test_SeleniumChromiumTestCase(self):

        with self.assertWarns(DeprecationWarning) as cm:
            SeleniumChromiumTestCase()

        self.assertEqual(
            str(cm.warning), "Use 'from django_tools.selenium.django import SeleniumChromiumStaticLiveServerTestCase' !"
        )

    def test_SeleniumFirefoxTestCase(self):

        with self.assertWarns(DeprecationWarning) as cm:
            SeleniumFirefoxTestCase()

        self.assertEqual(
            str(cm.warning), "Use 'from django_tools.selenium.django import SeleniumFirefoxStaticLiveServerTestCase' !"
        )

    def test_find_executable(self):

        with self.assertWarns(DeprecationWarning) as cm:
            with self.assertRaises(FileNotFoundError):
                find_executable("foobar")

        self.assertEqual(str(cm.warning), "Use 'from django_tools.selenium.utils import find_executable' !")

    def test_chromium_available(self):

        with self.assertWarns(DeprecationWarning) as cm:
            chromium_available()

        self.assertEqual(str(cm.warning), "Use 'from django_tools.selenium.chromedriver import chromium_available' !")

    def test_firefox_available(self):

        with self.assertWarns(DeprecationWarning) as cm:
            firefox_available()

        self.assertEqual(str(cm.warning), "Use 'from django_tools.selenium.geckodriver import firefox_available' !")



"""
    :created: 21.08.2018 by Jens Diemer
    :copyleft: 2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import logging
import time
from unittest import mock

from django.contrib.auth.models import AnonymousUser
from django.core.cache import cache
from django.test import RequestFactory, SimpleTestCase, override_settings

# https://github.com/jedie/django-tools
from django_tools.debug.delay import CacheDelay
from django_tools.unittest_utils.logging_utils import LoggingBuffer
from django_tools.unittest_utils.unittest_base import BaseTestCase
from django_tools.unittest_utils.user import TestUserMixin


class CacheDelayTests(SimpleTestCase):

    def setUp(self):
        super().setUp()
        cache.clear()

    def _get_request(self, url):
        rf = RequestFactory()
        request = rf.get(url)
        request.user = AnonymousUser()
        return request

    def test_delay(self):
        request = self._get_request("/foo/bar/?delay=2")

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            CacheDelay(
                key="delay", only_debug=False
            ).load(
                request,
                query_string="delay",
            )

        log.assert_messages([
            "INFO:django_tools.debug.delay:Add 'delay' value to cache",
            "WARNING:django_tools.debug.delay:Save 2 sec. from 'delay' for 'delay' into cache"
        ])

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
                CacheDelay(key="delay", only_debug=False).sleep()

        log.assert_messages(["WARNING:django_tools.debug.delay:Delay 2 sec. for 'delay'"])

        mock_method.assert_called_once_with(2)

    def test_not_set(self):
        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
                CacheDelay(key="test_not_set", only_debug=False).sleep()

        log.assert_messages(["DEBUG:django_tools.debug.delay:No delay for 'test_not_set' from cache"])

        mock_method.assert_not_called()

    @override_settings(DEBUG=False)
    def test_only_debug(self):
        request = self._get_request("/foo/bar/?delay=2")

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            CacheDelay(key="delay").load(
                request,
                query_string="delay",
            )

        log.assert_messages(["DEBUG:django_tools.debug.delay:Ignore ?delay, because DEBUG is not ON!"])

    def test_default_value(self):
        request = self._get_request("/foo/bar/?test_default_value")

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            CacheDelay(
                key="delay", only_debug=False
            ).load(
                request,
                query_string="test_default_value",
                default=123,
            )

        log.assert_messages([
            "INFO:django_tools.debug.delay:Add 'test_default_value' value to cache",
            "WARNING:django_tools.debug.delay:Save 123 sec. from 'test_default_value' for 'delay' into cache"
        ])

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
                CacheDelay(key="delay", only_debug=False).sleep()

        log.assert_messages(["WARNING:django_tools.debug.delay:Delay 123 sec. for 'delay'"])

        mock_method.assert_called_once_with(123)

    def test_delete_value(self):
        request = self._get_request("/foo/bar/?test_delete_value=3")

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            CacheDelay(
                key="delay", only_debug=False
            ).load(
                request,
                query_string="test_delete_value",
            )

        log.assert_messages([
            "INFO:django_tools.debug.delay:Add 'test_delete_value' value to cache",
            "WARNING:django_tools.debug.delay:Save 3 sec. from 'test_delete_value' for 'delay' into cache"
        ])

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            CacheDelay(
                key="delay", only_debug=False
            ).load(
                request,
                query_string="not_existing_key",
            )

        log.assert_messages(["DEBUG:django_tools.debug.delay:Delete 'delay' delay from cache"])


class SessionDelayTests(TestUserMixin, BaseTestCase):

    def setUp(self):
        super().setUp()
        self.client.logout()

    def test_delay(self):

        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
                response = self.client.get("/delay/?sec=0.02")

        self.assertResponse(
            response,
            must_contain=("django_tools_test_project.django_tools_test_app.views.delay_view",),
            must_not_contain=("error", "traceback"),
            status_code=200,
            messages=[],
            html=False,
            browser_traceback=True
        )

        log.assert_messages([
            "INFO:django_tools.debug.delay:Add 'sec' value to session",
            "WARNING:django_tools.debug.delay:Save 0.02 sec. from 'sec' for 'delay_view' into session",
            "WARNING:django_tools.debug.delay:Delay 0.02 sec. for 'delay_view'"
        ])

        mock_method.assert_called_once_with(0.02)

    def test_not_set(self):
        with LoggingBuffer(name="django_tools.debug.delay", level=logging.DEBUG) as log:
            with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
                response = self.client.get("/delay/")

        self.assertResponse(
            response,
            must_contain=("django_tools_test_project.django_tools_test_app.views.delay_view",),
            must_not_contain=("error", "traceback"),
            status_code=200,
            messages=[],
            html=False,
            browser_traceback=True
        )

        log.assert_messages(["DEBUG:django_tools.debug.delay:No delay for 'delay_view' from session"])

        mock_method.assert_not_called()

    def test_message(self):
        self.login(usertype="normal")
        with mock.patch.object(time, 'sleep', return_value=None) as mock_method:
            response = self.client.get("/delay/?sec=3")

        self.assertResponse(
            response,
            must_contain=("django_tools_test_project.django_tools_test_app.views.delay_view",),
            must_not_contain=("error", "traceback"),
            status_code=200,
            messages=["Use 3 sec. 'sec'"],
            html=False,
            browser_traceback=True
        )



# coding: utf-8

from __future__ import unicode_literals

from django_tools.unittest_utils.unittest_base import BaseUnittestCase
from django_tools.utils.html_utils import html2text


class TestHtmlUtils(BaseUnittestCase):
    def test_none(self):
        self.assertEqual(html2text(None), None)

    def test_empty(self):
        self.assertEqual(html2text(""), "")

    def test_text_only(self):
        self.assertEqual(html2text("foo"), "foo")

    def test_invalid_tags(self):
        self.assertEqual(html2text("<foo>the text</bar>"), "the text")



from django.contrib import admin

from django_tools_test_project.django_tools_test_app.models import PermissionTestModel

# https://github.com/jedie/django-tools
from django_tools.admin_tools.test_generator import generate_test_code


@admin.register(PermissionTestModel)
class PermissionTestModelAdmin(admin.ModelAdmin):
    pass


# Activate globally for all models:
admin.site.add_action(generate_test_code)



# coding: utf-8

import os
import sys

sys.path.insert(0, os.path.dirname(__file__))

SECRET_KEY = "Unittests"
ALLOWED_HOSTS = ["*"] # Allow any domain/subdomain

DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ":memory:"
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}

MIDDLEWARE_CLASSES = (
    'django_tools.dynamic_site.middleware.DynamicSiteMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',

    'django_tools.dynamic_site',
    'django_tools.dynamic_site.test_app',
)
SITE_ID = 1

ROOT_URLCONF = 'django_tools.dynamic_site.test_app.urls'

DEFAULT_CHARSET="utf-8"

USE_DYNAMIC_SITE_MIDDLEWARE = True



# coding: utf-8
import logging
import os
import warnings

from django.utils.translation import ugettext_lazy as _

# https://github.com/jedie/django-tools
from django_tools.unittest_utils.logging_utils import CutPathnameLogRecordFactory, FilterAndLogWarnings

print("Use settings:", __file__)

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

DEBUG = True

SECRET_KEY = "Unittests"
ALLOWED_HOSTS = ["*"]  # Allow any domain/subdomain

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, "..", "test_project_db.sqlite3"),
        # 'NAME': ":memory:"
    }
}

CACHES = {'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache', 'LOCATION': 'unique-snowflake'}}

MIDDLEWARE_CLASSES = (
    # https://github.com/jazzband/django-debug-toolbar/
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',

    # 'django_tools.dynamic_site.middleware.DynamicSiteMiddleware',
    'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
    'django_tools.middlewares.TracebackLogMiddleware.TracebackLogMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    'django.contrib.admindocs',
    'django.contrib.flatpages',
    'easy_thumbnails',
    'filer',  # for django_tools.unittest_utils.mockup

    # TODO:
    # # https://pypi.org/project/django-parler
    # 'parler',
    #
    # # https://pypi.org/project/django-ya-model-publisher/
    # 'publisher',
    'django_tools',
    'django_tools.local_sync_cache',
    # 'django_tools.dynamic_site',
    'django_tools_test_project.django_tools_test_app',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, "templates/"),
        ],
        'OPTIONS': {
            'loaders': [
                (
                    "django_tools.template.loader.DebugCacheLoader", (
                        'django.template.loaders.filesystem.Loader',
                        'django.template.loaders.app_directories.Loader',
                    )
                ),
            ],
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.template.context_processors.media',
                'django.template.context_processors.csrf',
                'django.template.context_processors.tz',
                'django.template.context_processors.static',
            ],
        },
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

# Default and fallback language:
# https://docs.djangoproject.com/en/1.11/ref/settings/#language-code
LANGUAGE_CODE = "en"

# http://django-parler.readthedocs.org/en/latest/quickstart.html#configuration
PARLER_LANGUAGES = {
    1: [
        {
            "name": _("German"),
            "code": "de",
            "fallbacks": [LANGUAGE_CODE],
            "hide_untranslated": False,
        },
        {
            "name": _("English"),
            "code": "en",
            "fallbacks": ["de"],
            "hide_untranslated": False,
        },
    ],
    "default": { # all SITE_ID"s
        "fallbacks": [LANGUAGE_CODE],
        "redirect_on_fallback": False,
    },
}

# https://docs.djangoproject.com/en/1.8/ref/settings/#languages
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGES = tuple([(d["code"], d["name"]) for d in PARLER_LANGUAGES[1]])

LANGUAGE_DICT = dict(LANGUAGES)  # useful to get translated name by language code

# http://docs.django-cms.org/en/latest/reference/configuration.html#std:setting-CMS_LANGUAGES
# CMS_LANGUAGES = PARLER_LANGUAGES

# http://django-parler.readthedocs.org/en/latest/quickstart.html#configuration
PARLER_DEFAULT_LANGUAGE_CODE = LANGUAGE_CODE

#==============================================================================

EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'

#==============================================================================

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

#==============================================================================

SITE_ID = 1

USE_DYNAMIC_SITE_MIDDLEWARE = True

ROOT_URLCONF = 'django_tools_test_project.django_tools_test_app.urls'

PASSWORD_HASHERS = (  # Speedup tests
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

#_____________________________________________________________________________

# Adds 'cut_path' attribute on log record. So '%(cut_path)s' can be used in log formatter.
# django_tools.unittest_utils.logging_utils.CutPathnameLogRecordFactory
logging.setLogRecordFactory(CutPathnameLogRecordFactory(max_length=50))

# Filter warnings and pipe them to logging system:
# django_tools.unittest_utils.logging_utils.FilterAndLogWarnings
warnings.showwarning = FilterAndLogWarnings()

#-----------------------------------------------------------------------------

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)8s %(pathname)s:%(lineno)-3s %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'null': {
            'class': 'logging.NullHandler',
        },
        'console': {
            'class': 'logging.StreamHandler',
            # 'formatter': 'simple'
            'formatter': 'verbose'
        },
    },
    'loggers': {
        "django_tools": {
            'handlers': [
                # 'null',
                'console'
            ],
            'level': 'DEBUG',
        },
        "django_tools.DynamicSite": {
            'handlers': [
                # 'null',
                'console'
            ],
            'level': 'DEBUG',
        },
    },
}



# coding: utf-8

"""
    some decorators
    ~~~~~~~~~~~~~~~

    from PyLucid decorators.

    :copyleft: 2009-2018 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import sys
import traceback
import warnings
from functools import wraps

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import Permission
from django.core.exceptions import PermissionDenied
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _


def check_permissions(superuser_only, permissions=()):
    """
    Protect a view and limit it to users witch are log in and has the permissions.
    If the user is not log in -> Redirect him to a log in view with a next_url back to the requested page.

    TODO: Add a log entry, if user has not all permissions.

    Usage:
    --------------------------------------------------------------------------
    @check_permissions(superuser_only=False, permissions=(u'appname.add_modelname', u'appname.change_modelname'))
    def my_view(request):
        ...
    --------------------------------------------------------------------------
    """
    assert isinstance(superuser_only, bool)
    assert isinstance(permissions, (list, tuple))

    def _inner(view_function):
        @wraps(view_function)
        def _check_permissions(request, *args, **kwargs):
            user = request.user

            if not user.is_authenticated():
                # FIXME: HttpResponseRedirect to admin login?
                msg = _("Permission denied for anonymous user. Please log in.")
                if settings.DEBUG:  # Usefull??
                    warnings.warn(msg)
                raise PermissionDenied(msg)

            if not user.has_perms(permissions):
                msg = "User %r has not all permissions: %r (existing permissions: %r)" % (
                    user,
                    permissions,
                    user.get_all_permissions(),
                )
                if settings.DEBUG:  # Usefull??
                    warnings.warn(msg)
                raise PermissionDenied(msg)
            return view_function(request, *args, **kwargs)

        # Add superuser_only and permissions attributes, so they are accessible
        # Used to build the admin menu
        _check_permissions.superuser_only = superuser_only
        _check_permissions.permissions = permissions

        return _check_permissions

    return _inner


def render_to(template_name=None, debug=False, **response_kwargs):
    """
    Based on the decorators from django-annoying.

    Example:

        @render_to('foo/template.html')
        def PyLucidPluginFoo(request):
            bar = Bar.object.all()
            return {'bar': bar}

    The view can also insert the template name in the context, e.g.:

        @render_to
        def PyLucidPluginFoo(request):
            bar = Bar.object.all()
            return {'bar': bar, 'template_name': 'foo/template.html'}
    """

    def renderer(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            context = function(*args, **kwargs)

            request = None
            for arg in args:
                if isinstance(arg, HttpRequest):
                    request = arg

            if not request:
                raise AttributeError("Can't get request object from function arguments!")

            if not isinstance(context, dict):
                if debug:
                    msg = ("renter_to info: %s (template: %r)" " has not return a dict, has return: %r (%r)") % (
                        function.__name__,
                        template_name,
                        type(context),
                        function.__code__,
                    )
                    messages.info(request, msg)
                return context

            template_name2 = context.pop("template_name", template_name)
            assert template_name2 != None, (
                "Template name must be passed as render_to parameter"
                " or 'template_name' must be inserted into context!"
            )

            response = render_to_response(
                template_name2, context, context_instance=RequestContext(request), **response_kwargs
            )

            if debug:
                messages.info(request, "render debug for %r (template: %r):" % (function.__name__, template_name2))
                messages.info(request, "local view context:", context)
                messages.info(request, "response:", response.content)

            return response

        return wrapper

    return renderer


def display_admin_error(func):
    """
    For temporary display errors in admin functions. e.g.:

        class MyAdmin(admin.ModelAdmin):

            @display_admin_error
            def bsp(obj):
                raise FooBar

            list_display = ('bsp',)
    """

    def wrapped(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as err:
            traceback.print_exc(file=sys.stderr)
            if settings.DEBUG:
                return "%s: %s" % (err.__class__.__name__, err)
            else:
                raise

    return wrapped


def warn_class_usage(message, category=DeprecationWarning):
    """
    create warnings on class usage.
    """

    def cls_wrapper(cls):
        class Wrapped(cls, object):
            def __init__(self, *args, **kwargs):
                warnings.warn(message, category)
                super(Wrapped, self).__init__(*args, **kwargs)

            def __new__(cls, *args, **kwargs):
                warnings.warn(message, category)
                return object.__new__(cls)

        return Wrapped

    return cls_wrapper


def warn_function_usage(message, category=DeprecationWarning):
    """
    create warnings on function usage.
    """

    def decorator(function):
        @wraps(function)
        def emit_warning(*args, **kwargs):
            warnings.warn(message, category)
            return function(*args, **kwargs)

        return emit_warning

    return decorator



# coding: utf-8

"""
    some utils around url
    ~~~~~~~~~~~~~~~~~~~~~

    :created: by Jens Diemer
    :copyleft: 2017 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

from django.http import QueryDict
from django.utils.encoding import force_bytes
from django.utils.six.moves.urllib.parse import quote


class GetDict(QueryDict):
    """
    Similar to origin django.http.QueryDict but:
        - always mutable
        - urlencode() output changed (see DocString there)
    """
    def __init__(self, *args, **kwargs):
        kwargs["mutable"]=True
        super(GetDict, self).__init__(*args, **kwargs)

    def _encode(self, k, v, safe=None):
        if safe is None:
            safe=""

        if v is None:
            return quote(k, safe)

        v = force_bytes(v, self.encoding)
        return '%s=%s' % ((quote(k, safe), quote(v, safe)))

    def urlencode(self, safe=None):
        """
        Same as django.http.QueryDict.urlencode() but:
            - None values will be a empty GET parameter "?empty" (Django: "?empty=")
            - output will be sorted (easier for tests ;)
        """
        output = []
        for k, list_ in sorted(self.lists()):
            k = force_bytes(k, self.encoding)
            output.extend(
                self._encode(k, v, safe)
                for v in sorted(list_)
            )
        return '&'.join(output)





# coding: utf-8

from __future__ import absolute_import, division, print_function

import logging

from django.test import TestCase

from django_tools.unittest_utils.logging_utils import LoggingBuffer


class TestTracebackLogMiddleware(TestCase):
    def test_exception_logging(self):
        with LoggingBuffer(name=None, level=logging.ERROR) as log:
            with self.assertRaises(Exception):
                self.client.get('/raise_exception/TestTracebackLogMiddleware/')

        messages = log.get_messages()
        print(messages)

        self.assertIn(
            "Exception on url: /raise_exception/TestTracebackLogMiddleware/",
            messages
        )
        self.assertIn(
            "Traceback (most recent call last):",
            messages
        )
        self.assertIn(
            'django_tools_test_app/views.py", line ',
            messages
        )
        self.assertIn(
            'in raise_exception',
            messages
        )
        self.assertIn(
            'raise Exception(msg)',
            messages
        )
        self.assertIn(
            "Exception: TestTracebackLogMiddleware",
            messages
        )
